use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileSize );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectExportDriver qw ( ExportObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use File::Temp qw ( tempdir tempfile );
use Readonly;
use Test::More;

Readonly my $ROOTALIAS => 'Test';
Readonly my $ROOTPATH => '/' .$ROOTALIAS;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval { testExportXML(); };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub testExportXML {
    my $TaxMethod = LoadObjectByPath( JoinObjectPath(['', $ROOTALIAS , 'TestShopAvaTax','TaxMethods','TaxMethodAvaTax']) );
    isa_ok( $TaxMethod, 'DE_EPAGES::AvaTax::API::Object::TaxMethodAvaTax', 'TaxMethodAvaTax' );

    my $dir = tempdir( CLEANUP => 1 );
    my ( undef, $filename ) = tempfile( DIR => $dir );

    eval {
        ExportObjects( $TaxMethod->parent->pathString, $filename, {}, [ $TaxMethod->alias ] );
        my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
        my $Struct = $ImportHandler->getXMLStructure( $filename, {} );
        cmp_ok( GetFileSize($filename), '>', 100, 'exportXML ok.' );
        ok( $Struct->existsElement('/TaxMethodAvaTax'), 'analyze structure' )
          or diag explain( $Struct->getRootElements() );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        fail( $Error->longMessage() );
    }
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
