## no critic( ProhibitExcessMainComplexity ProhibitNoWarnings )
## no critic( RequireBlockMap )
## no critic( RequireCorrectImports )
use utf8;
use 5.010;
use strict;
use warnings FATAL => 'all';
# use DE_EPAGES::Billsafe::UI::BasketOrder;
use DE_EPAGES::Billsafe::API::Util; # provides Util
use Test::More;

BEGIN { *Bubo:: = *DE_EPAGES::Billsafe::UI::BasketOrder:: }

# What is this file for?
# For testing housenumber matching, required for housenumber extraction.



# ==== subroutines ===========================================================

sub do_test {
    my( $s_in, $s_exp, $hn_exp ) = @_;
    $s_exp  //= $s_in;
    my( $street, $number ) = Util::extract_housenumber( $s_in );
    no warnings 'uninitialized';
    is $street, $s_exp, "got street $s_exp from [$s_in]";
    is $number, $hn_exp, "got housenumber $hn_exp from [$s_in]";
    return;
}

# Apply subroutines to data array.
# The objective is to make bulk changes to test data.
sub make_data_toggle {
    my( $data_ary, $invert, $revert ) = @_;
    my $state = 0;
    return sub {
        return $invert->( @$data_ary ) if $state++ & 1;
        return $revert->( @$data_ary );
    };
}



# ==== main ==================================================================

# Test data is contained in lists; each item is a row of three columns:
# 0: input ; 1: expected return street; 2: expeced return housenumber

# going to do wide characters
my $bldr = Test::Builder->new; # TB singleton
binmode $bldr->$_, ':encoding(UTF-8)' for qw/output failure_output todo_output/;


# ==== test German data ======================================================
my @lumped_together = (
    [ 'Schönberger Straße 20',              => 'Schönberger Straße', '20' ],
    [ 'Schönberger Straße 20a',             => 'Schönberger Straße', '20a' ],
    [ ' Schönberger Straße 1a !  ',         => 'Schönberger Straße', '1a !' ],
    [ "\tSchönberger Straße 1a !  ",        => 'Schönberger Straße', '1a !' ],
    [ "\nSchönberger Straße 1a !  ",        => 'Schönberger Straße', '1a !' ],
    [ "\n\nSchönberger Straße 1a !  ",      => 'Schönberger Straße', '1a !' ],
    [ "\r\n\r\nSchönberger Straße 1a",      => 'Schönberger Straße', '1a' ],
    [ 'Schönberger Straße 20 a',            => 'Schönberger Straße', '20 a' ],
    [ ' Schönberger Straße 20 a  ',         => 'Schönberger Straße', '20 a' ],
    [ ' Schönberger Straße 20 a !  ',       => 'Schönberger Straße', '20 a !' ],
    [ ' Schönberger Straße 20.a !  ',       => 'Schönberger Straße', '20.a !' ],
    [ ' Schönberger Straße 20,a !  ',       => 'Schönberger Straße', '20,a !' ],
    [ ' Schönberger Straße 20,a .  ',       => 'Schönberger Straße', '20,a .' ],
    [ ' Schönberger Straße 20,a.  ',        => 'Schönberger Straße', '20,a.' ],
    [ " Schönberger Straße 20\ta.  ",       => 'Schönberger Straße', '20 a.' ],
    [ "Schönberger Straße\t20\ta.  ",       => 'Schönberger Straße', '20 a.' ],
    [ "Schönberger\tStraße\t 20 \ta.  ",    => 'Schönberger Straße', '20 a.' ],
    [ "Schönberger\tStraße\t 20 \ta.  ",    => 'Schönberger Straße', '20 a.' ],
);
do_test @$_ for @lumped_together;

my $lump_toggle = make_data_toggle \@lumped_together,
    sub { for ( @_ ) { s/\b20\b/42-44/ for @$_ } },
    sub { for ( @_ ) { s/\b42-44\b/20/ for @$_ } };

$lump_toggle->(); do_test @$_ for @lumped_together; $lump_toggle->();

for ( @lumped_together ) { s/\bStraße\b/Str./ for @$_ }
0 and diag explain \@lumped_together;
do_test @$_ for @lumped_together;

$lump_toggle->(); do_test @$_ for @lumped_together; $lump_toggle->();

for ( @lumped_together ) { s/\sStr\./straße/ for @$_ }
0 and diag explain \@lumped_together;
do_test @$_ for @lumped_together;

$lump_toggle->(); do_test @$_ for @lumped_together; $lump_toggle->();

for ( @lumped_together ) { s/straße/ Weg/ for @$_ }
0 and diag explain \@lumped_together;
do_test @$_ for @lumped_together;

$lump_toggle->(); do_test @$_ for @lumped_together; $lump_toggle->();

for ( @lumped_together ) { s/Weg/W./ for @$_ }
0 and diag explain \@lumped_together;
do_test @$_ for @lumped_together;

$lump_toggle->(); do_test @$_ for @lumped_together; $lump_toggle->();

for ( @lumped_together ) { s/\bW\. /W./ for @$_ }
0 and diag explain \@lumped_together;
do_test @$_ for @lumped_together;

$lump_toggle->(); do_test @$_ for @lumped_together; $lump_toggle->();



# ==== test French data ======================================================
my @french = (
    [ '13 rue capucines',                   => 'rue capucines', '13' ],
    [ '13, rue capucines',                  => 'rue capucines', '13' ],
    [ '13- rue capucines',                  => 'rue capucines', '13' ],
    [ '13 - rue capucines',                 => 'rue capucines', '13' ],
    [ '13. rue capucines',                  => 'rue capucines', '13' ],
    [ '13: rue capucines',                  => 'rue capucines', '13' ],
    [ '13; rue capucines',                  => 'rue capucines', '13' ],
    [ '13a rue capucines',                  => 'rue capucines', '13a' ],
    [ '13 a rue capucines',                 => 'rue capucines', '13 a' ],
    [ '13.a rue capucines',                 => 'rue capucines', '13.a' ],
    [ '13.a, rue capucines',                => 'rue capucines', '13.a' ],
    [ '13.ab, rue capucines',               => 'rue capucines', '13.ab' ],
);
do_test @$_ for @french;

for ( @french ) { s/\brue\b/r./ for @$_ }
do_test @$_ for @french;

for ( @french ) { s/\br\.\b/r/ for @$_ }
do_test @$_ for @french;



# ==== test housenumber ranges ===============================================
my @ranges = ( # range indicator used here: plain old ASCII minus (dash)
    [ 'Herdentorsteinweg 44-45',            => 'Herdentorsteinweg', '44-45' ],
    [ 'Herdentorsteinweg 1a-1c',            => 'Herdentorsteinweg', '1a-1c' ],
    [ 'Herdentorsteinweg 1-2',              => 'Herdentorsteinweg', '1-2'   ],
    [ 'Herdentorsteinweg 1 -2',             => 'Herdentorsteinweg', '1 -2'  ],
    [ 'Herdentorsteinweg 1- 2',             => 'Herdentorsteinweg', '1- 2'  ],
    [ 'Herdentorsteinweg 1 - 2',            => 'Herdentorsteinweg', '1 - 2' ],
    [ 'Straße des 17 Juni 106-108'          => 'Straße des 17 Juni', '106-108'],
    [ 'Straße des 17. Juni 106-108'         => 'Straße des 17. Juni', '106-108'],
);

# test once using default range indicator (ASCII minus)
do_test @$_ for @ranges;

# run more tests using fancy new-fangled dashes
my @dashes = ( '-', "\x{2012}", "\x{2013}", "\x{2014}" );
for ( my $i = 1; $i <= $#dashes; $i++ ) {
    my $old = $dashes[ $i - 1 ];
    my $new = $dashes[ $i ];
    for ( @ranges ) { s/$old/$new/ for @$_ }
    do_test @$_ for @ranges;
}



# ==== Mannheim ==============================================================
my @Mannheim = (
    [ 'N2, 10'        => 'N2', '10', ], # Komma
    [ 'N2 10'         => 'N2', '10', ],
    [ 'N 2, 10'       => 'N 2', '10', ],
    [ 'N 2 10'        => 'N 2', '10', ],
);
do_test @$_ for @Mannheim;
for ( @Mannheim ) { s/,/;/ for @$_ } # Semikolon
do_test @$_ for @Mannheim;
for ( @Mannheim ) { s/;/:/ for @$_ } # Doppelpunkt
do_test @$_ for @Mannheim;



# ==== test some edge cases ==================================================
my @edge_cases = (
    [ '',                                       '', ''],
    [ undef,                                    '', ''],
    [ 99,                                       '99', ''],
    [ 'Gurkenchaussee',                         'Gurkenchaussee', ''],
    [ 'Gurkenchaussee jawoll! -99',             'Gurkenchaussee jawoll!', '-99'],
);
do_test @$_ for @edge_cases;

done_testing;
