use strict;
use Test::More tests => 18;
use DateTime;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;

sub Main {

    ok( 1, 'init');

    # date time
    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'de_DE');
    ok( $Formatter->parse_datetime('12.04.2004','date','medium')->strftime('%d.%m.%Y') eq '12.04.2004', 'de date parse');
    ok( $Formatter->format_datetime($Formatter->parse_datetime('12.04.2004','date','medium'),'date','medium') eq '12.04.2004', 'de date format');
    ok( $Formatter->format_datetime($Formatter->parse_datetime('12','time', '%H'),'datetime','medium') eq '01.01.1 12:00:00', 'de time format');

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'de_DE');
    my $DatetimeFormat  = '%d.%m.%Y %H:%M';
    my $DatetimeType    = 'datetime';
    ok( $Formatter->parse_datetime('12.04.2004 11:24',$DatetimeType, $DatetimeFormat)->strftime($DatetimeFormat) eq '12.04.2004 11:24', 'de datetime parse');
    ok( $Formatter->format_datetime($Formatter->parse_datetime('12.04.2004 11:24',$DatetimeType, $DatetimeFormat), $DatetimeType, $DatetimeFormat) eq '12.04.2004 11:24', 'de datetime format');

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'en_GB');
    ok( $Formatter->parse_datetime('12 April 2004','date', 'medium')->strftime('%d.%m.%Y') eq '12.04.2004', 'en_GB parse');
    ok( $Formatter->format_datetime($Formatter->parse_datetime('12 April 2004', 'date', 'medium'), 'date', 'medium') eq '12 Apr 2004', 'en_GB format');

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'en_GB');
    ok( $Formatter->parse_datetime('12 April 2004 11:24:00', 'datetime', 'medium' )->strftime('%d.%m.%Y %H:%M') eq '12.04.2004 11:24', 'en_GB datetime parse');
    ok( $Formatter->format_datetime($Formatter->parse_datetime('12 April 2004 11:24:00', 'datetime', 'medium' ), 'datetime', 'medium' ) eq '12 Apr 2004 11:24:00', 'en_GB datetime format');

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    ok( $Formatter->format_datetime(DateTime->now, 'datetime', 'medium'), 'locale format');
    ok( $Formatter->parse_datetime($Formatter->format_datetime(DateTime->now, 'datetime', 'medium'), 'datetime', 'medium'), 'locale parse');

    # custom date time format
    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'de_DE');
    ok( $Formatter->parse_datetime('2004-04-12','date','%Y-%m-%d')->strftime('%d.%m.%Y') eq '12.04.2004', 'TrustedShops-MoneyBackWarrenty-DelieryDateFormat parse');
    ok( $Formatter->format_datetime($Formatter->parse_datetime('12.04.2004','date','medium'),'date','%Y-%m-%d') eq '2004-04-12', 'TrustedShops-MoneyBackWarrenty-DelieryDateFormat format');

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'de_DE');
    $DatetimeFormat  = '%Y-%m-%d %H:%M';
    $DatetimeType    = 'datetime';
    ok( $Formatter->parse_datetime('2004-04-12 11:24',$DatetimeType, $DatetimeFormat)->strftime('%d.%m.%Y %H:%M') eq '12.04.2004 11:24', 'Custom DateTimeFormat parse');
    ok( $Formatter->format_datetime($Formatter->parse_datetime('2004-04-12 11:24',$DatetimeType, $DatetimeFormat), $DatetimeType, '%d.%m.%Y %H:%M') eq '12.04.2004 11:24', 'Custom DateTimeFormat format');

    eval { $Formatter->format_datetime( DateTime->now, 'date_tz') };
    ok( ExistsError(), 'Unknown format' );
    is( GetError()->code, 'FORMAT_UNKNOWN', 'Error code for unknown format' ) or diag GetError()->longMessage;
}

RunScript(Sub=>\&Main, Debug=>1);
