use Test::Simple tests => 8;

use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw(MergeIniFiles CopyFile DeleteFile
                                  CompareMD5File GetFileMD5Digest);

my $TargetFile = 'Ini/merge-result.conf';
my $AddFile = 'Ini/merge+.conf';

sub Main {

    my ($Result, $ExpectedFile);

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile);
    $ExpectedFile = 'Ini/merge-expected.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(undef, $ExpectedFile));

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile, undef, {replace => 'sections'});
    $ExpectedFile = 'Ini/merge-expected-replacesection.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(', undef, {replace => "sections"}', $ExpectedFile));

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile, [qw(Section1)]);
    $ExpectedFile = 'Ini/merge-expected-editsection.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(', [qw(Section1)]', $ExpectedFile));

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile, [qw(Section1)], {replace => 'parameters'});
    $ExpectedFile = 'Ini/merge-expected-replaceparameters.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(', [qw(Section1)], {replace => "parameters"}', $ExpectedFile));

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile, [qw(Section1)], {replace => 'all'});
    $ExpectedFile = 'Ini/merge-expected-replaceall.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(', [qw(Section1)], {replace => "all"}', $ExpectedFile));

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile, [qw(Section1)], {replace => 'all', delcomments => 'all'});
    $ExpectedFile = 'Ini/merge-expected-replace-delcomments.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(', [qw(Section1)], {replace => "all", delcomments => "all"}', $ExpectedFile));

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile, [qw(Section1)], {delcomments => '#'});
    $ExpectedFile = 'Ini/merge-expected-delhash.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(', [qw(Section1)], {delcomments => "#"}', $ExpectedFile));

    CopyFile('Ini/merge.conf', $TargetFile);
    MergeIniFiles($TargetFile, $AddFile, [qw(Section1)], {delcomments => ';'});
    $ExpectedFile = 'Ini/merge-expected-delsemicolon.conf';
    $Result = CompareMD5File($TargetFile, GetFileMD5Digest($ExpectedFile));
    ok( $Result, _fString(', [qw(Section1)], {delcomments => ";"}', $ExpectedFile));

    DeleteFile($TargetFile);
}

sub _fString {
    my ($Args, $ExpectedFile) = @_;
    return "MergeIniFiles($TargetFile, $AddFile$Args) cmp $ExpectedFile";
}

RunScript(Sub=>\&Main, Debug=>1);
