use Test::More tests => 15;

use strict;
use Config;
use Time::HiRes qw(time);
use Proc::Background;
use DE_EPAGES::Core::API::Process qw(
    GetProcessMemory
    GetCurrentProcessID
    SetProcessPriority
    PurgeBackgroundProcesses
    WaitForBackgroundProcesses
);

# suppress warings from Win32::API::Struct
local $SIG{__WARN__} = sub {
    my ($Message) = @_;
    print STDERR $Message unless $Message =~ /Win32\/API\/Struct/;
};

my $PID = GetCurrentProcessID();
ok( 0 != $PID, 'GetCurrentProcessID' );

my $MemUsage = GetProcessMemory( $PID );
ok( $MemUsage > 5000000, 'GetProcessMemory' );

SetProcessPriority( $PID, 'ABOVE_NORMAL' );
ok ( 1, 'SetProcessPriority' );

# test PurgeBackgroundProcesses
foreach my $Timeout (undef, 4, 2) {
    my $aProcessList = [ map { _NewProcess( $_ ) } (2,4,6) ];
    sleep 1;

    my @ExpectedRemaining = (3,2,1,0);
    @ExpectedRemaining = (3,2,0) if $Timeout == 4;
    @ExpectedRemaining = (3,0) if $Timeout == 2;
    foreach (@ExpectedRemaining) {
        $aProcessList = PurgeBackgroundProcesses($aProcessList, { timeout => $Timeout });
        ok( scalar @$aProcessList == $_, "PurgeBackgroundProcesses (timeout: $Timeout, parallel: $_)" );
        sleep 2;
    }
}

# test WaitForBackgroundProcesses
foreach my $NumParallel (1,2,3) {
    my $aProcessList = [];
    my %SleepProcesses;
    my $StartTime = time;
    note "WaitForBackgroundProcesses( 'sleep 2/4/6/8', { parallel => $NumParallel })";
    foreach my $Seconds (2,4,6,8) {
        my $Proc = _NewProcess( $Seconds );
        push @$aProcessList, $Proc;
        $SleepProcesses{$Seconds} = $Proc;
        _ShowProcesses(\%SleepProcesses, $StartTime);
        $aProcessList = WaitForBackgroundProcesses($aProcessList, { parallel => $NumParallel });
    }
    _ShowProcesses(\%SleepProcesses, $StartTime);
    WaitForBackgroundProcesses($aProcessList);
    _ShowProcesses(\%SleepProcesses, $StartTime);
    my $Duration = int(time - $StartTime);
    my $ExpectedDuration = 2+4+6+8;
    $ExpectedDuration = 4+8 if $NumParallel == 2;
    $ExpectedDuration = 2+8 if $NumParallel == 3;
    ok( $Duration == $ExpectedDuration, "duration: $Duration" );
}

sub _ShowProcesses {
    my ($hProcesses, $StartTime) = @_;

    return 0 unless scalar grep { $_->alive } values %$hProcesses;

    note sprintf '%.3f : %s', time - $StartTime, join(' ',
        map { sprintf '%2d (%s)', $_, ($hProcesses->{$_}->alive ? sprintf('%5d', $hProcesses->{$_}->pid) : '     ') }
        sort { $a <=> $b }
        keys %$hProcesses
    );

    return 1;
}

sub _NewProcess {
    my ($Duration) = @_;

    return $Config{'osname'} eq 'MSWin32'
        ? Proc::Background->new("perl -e  \"sleep $Duration\"")
        : Proc::Background->new("sleep $Duration");
}