use strict;
use Test::More;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/Scripts");
use Perl::Critic::Policy::DE_EPAGES::Style::RequireCommentForExportedFunctions;

sub Main {    ## no critic (DE_EPAGES::Style::ProhibitFunctionSize)

    my $critic = Perl::Critic->new(
        -profile  => 'NONE',
        -severity => 5
    );
    ok( defined $critic, 'Perl::Critic initialized' );
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::Style::RequireCommentForExportedFunctions' );
    my @policies = $critic->policies();
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Style::RequireCommentForExportedFunctions' } @policies;
    ok( $policyLoaded, 'Policy loaded' );

    my @violations;
    my $sourceCode;

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::Payment
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  tTest
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package tTest::PayPalExpress::API::Payment;
use base Exporter;
use strict;

our @EXPORT_OK  =  qw (
    CheckoutRequest
    CheckoutRequestDetails
);

sub CheckoutRequest {

}

sub CheckoutRequestDetails {

}
1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 2, 'function documentation missing' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::PaymenttTest
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  tTest
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package tTest::PayPalExpress::API::PaymenttTest;
use base Exporter;
use strict;

our @EXPORT_OK  =  qw (
    tTestCheckoutRequest
);

#========================================================================================
# function     tTestCheckoutRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestCheckoutRequest( $Input1 );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# return       $ReturnVariable | ReturnDescription | any
#========================================================================================
sub tTestCheckoutRequest {
    my $self = shift;
    my ($Input1, $Input2) = @_;
}

1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 1, 'input not described in comment' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::tTestPayment
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  Test
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package tTest::PayPalExpress::API::tTestPayment;
use base Exporter;
use strict;

our @EXPORT_OK  =  qw (
    tTestCheckoutRequest
);

#========================================================================================
# function     tTestCheckoutRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestCheckoutRequest( $Input1, $Input2, $Input3 );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Input1 | Input1Description | string
# input        $Input2 | Input1Description | string
# input        $Input3 | Input1Description | string
# return       $ReturnVariable | ReturnDescription | object
#========================================================================================
sub tTestCheckoutRequest {
    my $self = shift;
    my ($Input1, $Input2, $Input3) = @_;
}

1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 0, 'variables ok' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::tTestPayment2
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package tTest::PayPalExpress::API::tTestPayment;
use base Exporter;
use strict;

our @EXPORT_OK  =  qw (
    tTestCheckoutRequest
);

#========================================================================================
# function     tTestCheckoutRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestCheckoutRequest( $Input1, $Input2, $Input3 );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Input1 | Input1Description | string
# input        $Input2 | Input1Description | string
# input        $Input3 | Input1Description | string
#========================================================================================
sub tTestCheckoutRequest {
    my $self = shift;
    my ($Input1, $Input2, $Input3) = @_;
}

1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 1, 'package description empty' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::tTestPayment
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  tTest
#----------------------------------------------------------------------------------------
# example      
#========================================================================================
package tTest::PayPalExpress::API::tTestPayment;
use base Exporter;
use strict;
        
our @EXPORT_OK  =  qw (
    tTestCheckoutRequest
);

#========================================================================================
# function     tTestCheckoutRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestCheckoutRequest( $Input1, $Input2, $Input3 );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Input1 | Input1Description | string
# input        $Input2 | Input1Description | string
# input        $Input3 | Input1Description | string
#========================================================================================
sub tTestCheckoutRequest {
    my $self = shift;
    my ($Input1, $Input2, $Input3) = @_;
    return $Input1;
}

1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 1, 'return documentation missing.' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::tTestPayment
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  tTest
#----------------------------------------------------------------------------------------
# example      
#========================================================================================
package tTest::PayPalExpress::API::tTestPayment;
use base Exporter;
use strict;
        
our @EXPORT_OK  =  qw (
    tTestCheckoutRequest
);

#========================================================================================
# function     tTestCheckoutRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestCheckoutRequest( $Input1, $Input2, $Input3 );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Input1 | Input1Description | string
# input        $Input2 | Input1Description | string
# input        $Input3 | Input1Description | string
#========================================================================================
sub tTestCheckoutRequest {
    my $self = shift;
    my ($Input1, $Input2, $Input3) = @_;
    return if ($Input1);
}

1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 0, 'return plus conditional statement.' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::tTestPayment
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package tTest::PayPalExpress::API::tTestPayment;
use base Exporter;
use strict;

our @EXPORT_OK  =  qw (
    tTestCheckoutRequest
);

#========================================================================================
# function     tTestCheckoutRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestCheckoutRequest( $Input1, $Input2, $Input3 );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Input1 | Input1Description | string
# input        $Input2 | Input1Description | string
# output       $hInput3 | Output Description | string
# return       $ReturnVariable | ReturnDescription | object
#========================================================================================
sub tTestCheckoutRequest {
    my $self = shift;
    my ($Input1, $Input2, $hInput3) = @_;
}

1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 0, 'output variable test' ) or diag(@violations);

    $sourceCode = <<'EOL';
#========================================================================================
# package      tTest::Core::API::File
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides file and directory access.
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::Core::API::File qw( GetFileContent WriteFile ...);
#========================================================================================
package tTest::Core::API::File;

use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    tTestDeleteDirectory
);

#========================================================================================
# function     tTestDeleteDirectory
#----------------------------------------------------------------------------------------
# syntax       $Deleted = tTestDeleteDirectory($Path, $hFlags);
#----------------------------------------------------------------------------------------
# example      $Deleted = tTestDeleteDirectory('C:\\tTest',{'recursive' => 1});
#----------------------------------------------------------------------------------------
# description  Removes the directory. Throws an error if the directory cannot be
#               deleted, for example because it contains files or sub directories and the
#               recursive flag is not set. If the directory did not exist before,
#               then the function does nothing.
#----------------------------------------------------------------------------------------
# input        $Path | directory | string
# input        $hFlags | flags to modify the behaviour of DeleteDirectory.
#                         The following flag can be used:
#                       <ul>
#                           <li>recursive - delete directory recursive with
#                               all contents like rm -rf (default = 0 ) </li>
#                           <li>move - instead of deleting the directory files, move it
#                               to the _TRASH_ folder in EPAGES_STATIC, EPAGES_STORES
#                               or EPAGES_WEBROOT (default=0, implies recursive)</li>
#                       </ul>
#                       | hash ref
# input        $TrashRoot | trash directory | string (optional)
# return       $Deleted | 1 directory was removed , 0 directory doesnt exists | boolean
#========================================================================================
sub tTestDeleteDirectory ($;$$);
sub tTestDeleteDirectory ($;$$) {
    my ($Path,$hFlags,$TrashRoot) = @_;
    return 0 unless -e $Path;

    # workaround for EPG-4777: delete directory containing files with special characters
    $Path = encode('iso-8859-1', $Path);
}
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 0, 'function prototype test' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::PayPalExpress::API::tTestPayment
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  tTest
#----------------------------------------------------------------------------------------
# example      
#========================================================================================
package tTest::PayPalExpress::API::tTestPayment;
use base Exporter;
use strict;
        
our @EXPORT_OK  =  qw (
    tTestCheckoutRequest
);

#========================================================================================
# function     tTestCheckoutRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestCheckoutRequest( $Input1, $Input2, $Input3 );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Input1 | Input1Description | string
# input        $Input2 | Input1Description | string
# input        $Input3 | Input1Description | string
# return       $Input1
#========================================================================================
sub tTestCheckoutRequest {
    my $self = shift;
    my $Input1 = shift;
    my ($Input2, $Input3) = @_;
    return $Input1;
}

1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 0, 'using shift in function arguments' ) or diag(@violations);

    $sourceCode = << 'EOL';
#========================================================================================
# package      tTest::Core::API::tTestLog
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides an interface for logging. It's actually a wrapper
#               for the Log::Log4perl module.
#========================================================================================
package tTest::Core::API::tTestLog;
use base Exporter;
use strict;

our @EXPORT_OK = qw(
tTestLogDebug
);

#========================================================================================
# function     tTestLogDebug
# state        public
#----------------------------------------------------------------------------------------
# syntax       tTestLogDebug( $Name, $rStructure);
# syntax       tTestLogDebug( $Name, $rStructure, $Category );
# example      tTestLogDebug( 'ProductInfo', \%ProductInfo, 'Product');
#----------------------------------------------------------------------------------------
# description  Dumps the data structure $rStructure to the log file if the log level for
#               category $Category (or current package) is 'DEBUG'.
#----------------------------------------------------------------------------------------
# input        $Name | name of data structure for identification in the log file | string
# input        $rStructure | the object | scalar (usually hash or array ref)
# input        $Category | debug category (default: calling package) | string
#========================================================================================
sub tTestLogDebug ($$;$) {
    my ($Name, $Structure, $Category) = @_;

    local $Log: RequireCommentForExportedFunctions.t,v $
    local Revision 1.11  2011/09/21 08:05:29  agrohmann
    local merge B_patch_6_12
    local
    local Revision 1.10.6.1  2011/09/20 10:11:05  otrosien
    local Critic: Fix detection of anonymous subs, some replacements not to confuse findUseProblems.pl
    local
    local Revision 1.10  2011/03/23 13:05:01  otrosien
    local issue EPG-17977: Fix API documentation errors found by Perl::Critic
    local https://jira.epages.de/browse/EPG-17977
    local
    local - fix false positive in anonymous sub
    local - fix false positive when having multiple syntax lines
    local - removed usage of newlines, because it breaks in eclipse
    localLog::Log4perl::caller_depth + 1;
    GetLog($Category)->debug(sub {
        local $Data::Dumper::Pad        = '';
        local $Data::Dumper::Varname    = $Name;
        local $Data::Dumper::Indent     = 1;
        local $Data::Dumper::Deepcopy   = 1;
        local $Data::Dumper::Sortkeys   = 1;
        local $Data::Dumper::Purity     = 0;
        local $Data::Dumper::Maxdepth   = 6;
        return Dumper($Structure);
    });
    my $cSub = sub {
        my ($Bla, $Blu) = @_;
        return $Bla . $Blu;
    };
    return;
}
1;
EOL

    @violations = $critic->critique( \$sourceCode );
    is( scalar @violations, 0, 'catch false positives using anonymous sub' ) or diag(@violations);

}

RunScript( Sub => \&Main );
done_testing();
