use Test::More tests => 20;

use strict;
use Config;
use Config::IniFiles;
use File::Basename  qw ( basename dirname );

use DE_EPAGES::Core::API::File qw( DeleteDirectory CompareMD5File GetFileMD5Digest );
use DE_EPAGES::Core::API::Script qw( RunScript );

sub Main {
    my $PerlExe = $Config{osname} eq 'MSWin32' ? $^X : $ENV{PERL};
    my $ConfigFile = 'Ini/Scheduler.conf';

    my %Ini;
    tie %Ini, 'Config::IniFiles', (-file => $ConfigFile)
        or die "$ConfigFile: Can't read file: $!";

    DeleteDirectory('log', {recursive => 1});
    foreach my $Scheduler (keys %Ini) {
        next if $Scheduler eq 'LOOPS';
        my $Command = "$PerlExe $ENV{EPAGES}/bin/wrapScheduler.PL -ini $ConfigFile -var log $Scheduler";
        note "command: $Command";
        my $Output = `$Command`;
        my $Result = 0;
        my $RunFile;
        my $i = 0;
=for example: $Output
  6285: /srv/epages/eproot/Perl/bin/perl sleep.PL  2 > "/tmp/ACCPL9NS6c/build-vm-lin2.Parallel3Loop.run-000001" 2>&1
 0.124: 0( 6285)
  6293: /srv/epages/eproot/Perl/bin/perl sleep.PL  4 > "/tmp/ACCPL9NS6c/build-vm-lin2.Parallel3Loop.run-000002" 2>&1
 0.135: 0( 6285) 1( 6293)
  6297: /srv/epages/eproot/Perl/bin/perl sleep.PL  6 > "/tmp/ACCPL9NS6c/build-vm-lin2.Parallel3Loop.run-000003" 2>&1
 0.144: 0( 6285) 1( 6293) 2( 6297)
  6414: /srv/epages/eproot/Perl/bin/perl sleep.PL  8 > "/tmp/ACCPL9NS6c/build-vm-lin2.Parallel3Loop.run-000004" 2>&1
 3.097: 0(     ) 1( 6293) 2( 6297) 3( 6414)
 5.128: 0(     ) 1(     ) 2( 6297) 3( 6414)
=cut
        foreach my $Line (split(/\n/, $Output)) {
            $RunFile = (split(/"/, $Line))[1] unless $RunFile; # "
            $RunFile =~ s,\.run-[0-9]+$,.run,;
            next unless $Line =~ / 0\(/;
            last if $Scheduler eq 'NoLoop';

            my $ParallelRunning = $Line =~ s/\( *[0-9]//g;
            $Result = $ParallelRunning if $ParallelRunning > $Result;
            $i++;
            note "scheduler $Scheduler, period $i, parallel $ParallelRunning";
        }
        my $Expected = $Scheduler;
        $Expected =~ s/[^0-9]//g;
        $Expected = $Scheduler eq 'NoLoop' ? 0 : 1 unless $Expected;
        ok( $Result eq $Expected, "scheduler $Scheduler, max. parallel $Result (expected: $Expected)");

        my $ExpectedFile = "Ini/scheduler-$Scheduler-expected.txt";
        $Result = CompareMD5File($ExpectedFile, GetFileMD5Digest($RunFile));
        ok( $Result, "CompareMD5File($ExpectedFile, GetFileMD5Digest($RunFile))");

        my $BaseName = basename($RunFile, '.run');
        my $TempDir = dirname($RunFile);
        ok( -s "$TempDir/$BaseName.mail", "-s $TempDir/$BaseName.mail");
        ok( -s "log/$BaseName.log", "-s log/$BaseName.log");
        ok( -s "log/$BaseName.pid", "-s log/$BaseName.pid");
        DeleteDirectory($TempDir, {recursive => 1});
    }
    DeleteDirectory('log', {recursive => 1});
}

RunScript(Sub=>\&Main, Debug=>1);
