use Test::More tests => 2;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use SOAP::Lite;

#--------------------------------------------------------------------------------------
sub testParseDHLSoapResponse {

my $xml = <<END_XML;
<env:Envelope xmlns:popws='https://www.dhl.de/popweb/gw/ws/schema/1.0/popws' xmlns:popsc='https://www.dhl.de/popweb/gw/ws/schema/1.0/popsc' xmlns:popbase='https://www.dhl.de/popweb/gw/ws/schema/1.0/popbase' xmlns:env='http://schemas.xmlsoap.org/soap/envelope/' xmlns:popps='https://www.dhl.de/popweb/gw/ws/schema/1.0/popps'>
  <env:Header />
  <env:Body env:encodingStyle=''>
    <popws:LoadBuyedShoppingCartResponse version='1.1.0'>
      <popws:Status status='0' />
      <popws:DownloadShoppingCart appletUrl='https://secure.micromata.de/popweb/gw/pages/StandaloneAwtPrintClient.gspt?MARKETPLACE=DHLANON&amp;scid=HSTLEVU5W' shoppingCartId='HSTLEVU5W' downloadPageUrl='https://secure.micromata.de/popweb/DownloadPostagePDF.do?scid=HSTLEVU5W'>
        <popsc:LabelItem labelId='MGFZYTY3ST' shipmentId='962006579699' customerReference='13875' productId='PAK10.DEU' awtDownloadUrl='https://secure.micromata.de/popweb/gw/pages/StandaloneAwtPrintClient.gspt?MARKETPLACE=DHLANON&amp;scid=HSTLEVU5W&amp;labelId=MGFZYTY3ST' pdfDownloadUrl='https://secure.micromata.de/popweb/getPdf/jan_spanier-Paket_bis_10_kg-MGFZYTY3ST.pdf' routingCode='20355030002005'>
          <popsc:Price>
            <popsc:PriceComponents vat='0' bruttoAmount='5.90' />
            <popsc:PriceComponents vat='19' bruttoAmount='0.10' />
          </popsc:Price>
          <popsc:SenderAddress name2='Max Mustermann' name1='Milestones GmbH' street='Musterstrasse' houseNumber='1' countryCode='DEU' email='' city='Munster' phoneNumber='' postalCode='12345' />
          <popsc:ReceiverAddress name1='jan spanier' street='pilatuspool' houseNumber='2' countryCode='DEU' email='' city='hamburg' phoneNumber='' postalCode='20355' />
          <popsc:Dimension />
        </popsc:LabelItem>
        <popsc:LabelItem labelId='KEAHKZW9LR' shipmentId='887002638568' productId='PAECK.DEU' awtDownloadUrl='https://secure.micromata.de/popweb/gw/pages/StandaloneAwtPrintClient.gspt?MARKETPLACE=DHLANON&amp;scid=HSTLEVU5W&amp;labelId=KEAHKZW9LR' customerReference='13884' pdfDownloadUrl='https://secure.micromata.de/popweb/getPdf/jan_spanier-Paeckchen_bis_2_kg-KEAHKZW9LR.pdf' routingCode='20355030002005'>
          <popsc:Price>
            <popsc:PriceComponents vat='0' bruttoAmount='3.90' />
          </popsc:Price>
          <popsc:SenderAddress name2='Max Mustermann' name1='Milestones GmbH' street='Musterstrasse' houseNumber='1' countryCode='DEU' email='' city='Munster' phoneNumber='' postalCode='12345' />
          <popsc:ReceiverAddress name1='jan spanier' street='pilatuspool' houseNumber='2' countryCode='DEU' email='' city='hamburg' phoneNumber='' postalCode='20355' />
          <popsc:Dimension />
        </popsc:LabelItem>
        <popsc:PaymentInfo status='' statusDate='2010-01-13T15:19:14+01:00' paymentMethod='DoExpressCheckoutPayment' confirmStatus='PAID' paymentId='PAYPALSE' transactionId1='0M146031E6828230H' paymentCode='Success' transactionId2='3633556c677a6' />
      </popws:DownloadShoppingCart>
    </popws:LoadBuyedShoppingCartResponse>
  </env:Body>
</env:Envelope>
END_XML

    my $rLoadContent = SOAP::Deserializer->deserialize($xml);
    my $ResultStr = '';

    my $i = 0;
    foreach my $LabelItem ($rLoadContent->dataof('//LoadBuyedShoppingCartResponse/DownloadShoppingCart/LabelItem')) {
        $i++;
        my $ShipmentId      = $LabelItem->attr->{'shipmentId'};
        my $LabelId         = $LabelItem->attr->{'labelId'};
        my $ProductId       = $LabelItem->attr->{'productId'};
        my $RoutingCode     = $LabelItem->attr->{'routingCode'};
        foreach my $PriceComponent ($rLoadContent->dataof("//LoadBuyedShoppingCartResponse/DownloadShoppingCart/[$i]/Price/*")) {
            my $Vat = $PriceComponent->attr()->{'vat'};
            my $BruttoAmount = $PriceComponent->attr()->{'bruttoAmount'};

            $ResultStr .= $Vat.' - '.$BruttoAmount.' // ';
        }

        $ResultStr .= $ShipmentId.' // '.$LabelId.' // '.$ProductId.' // '.$RoutingCode.' // ';

    }

    is($ResultStr,'0 - 5.90 // 19 - 0.10 // 962006579699 // MGFZYTY3ST // PAK10.DEU // 20355030002005 // 0 - 3.90 // 887002638568 // KEAHKZW9LR // PAECK.DEU // 20355030002005 // ',  'test ParseDHLSoapResponse');
}

#--------------------------------------------------------------------------------------
RunScript(
    'Sub' => sub {
        ok( 1, 'test started' );
        testParseDHLSoapResponse;
        return;
    }
);
