use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DHL::API::Constants qw ( BUYONLINE_ATTRIBUTE_ID );
use DE_EPAGES::DHL::API::Tools;
use DE_EPAGES::DHL::DAL::DHLProducts qw ( DAL_GetDHLProducts );
use Test::More;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # tests
            TestGetProduct(
                'PAK02.VERSI.DEU',
                {
                    'Length'     => 60,
                    'Width'      => 30,
                    'Height'     => 15,
                    'LWH'        => 50,
                    'Weight'     => 1.5,
                    'Country'    => 'DEU',
                    'Attributes' => ['VERSI'],
                },
                { 'DHLProductID' => 'PAK02.VERSI.DEU', }
            );

            # tests
            TestGetProduct(
                'PAK02.DEU',
                {
                    'Length'     => 60,
                    'Width'      => 30,
                    'Height'     => 15,
                    'LWH'        => 50,
                    'Weight'     => 1.5,
                    'Country'    => 'DEU',
                    'Attributes' => ['Tracking'],
                },
                { 'DHLProductID' => 'PAK02.DEU', }
            );

            # tests
            TestGetProduct(
                'PAK30.VERSI.DEU',
                {
                    'Length'     => 60,
                    'Width'      => 30,
                    'Height'     => 15,
                    'LWH'        => 50,
                    'Weight'     => 30,
                    'Country'    => 'DEU',
                    'Attributes' => ['VERSI'],
                },
                {
                    'DHLProductID'          => 'PAK31.VERSI.DEU',
                    'OverallPriceAmountEUR' => '16.4',
                }
            );

            done_testing();
        }
    );
}

sub TestGetProduct {
    my ( $TestPrefix, $hInput, $hTestParams ) = @_;
    eval {
        my $ahDHLProducts = DAL_GetDHLProducts( @$hInput{qw(Length Width Height LWH Weight Country)}, BUYONLINE_ATTRIBUTE_ID );
        ## no critic (ProtectPrivateSubs)
        $ahDHLProducts = DE_EPAGES::DHL::API::Tools::_CheckValidityOfDHLProductIDs($ahDHLProducts);
        $ahDHLProducts = DE_EPAGES::DHL::API::Tools::_ExistsAttributesForDHLProduct( $ahDHLProducts, $hInput->{'Attributes'} );
        cmp_ok( scalar @$ahDHLProducts, '>', 0, $TestPrefix . ': ' . scalar @$ahDHLProducts . ' results.' );
        my $hDHLProduct = $ahDHLProducts->[0];
        is( $hDHLProduct->{$_}, $hTestParams->{$_}, $TestPrefix . ': ' . $_ ) foreach ( keys %$hTestParams );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        fail( $Error->longMessage );
    }
}

RunScript( Sub => \&Main );

1;
