use strict;
use Test::More tests => 3;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Diagnostics::API::Dependencies;
use DE_EPAGES::Diagnostics::API::DrawDependencies;

sub TestDependencies {

    my $Cartridge =
      DE_EPAGES::Diagnostics::API::Dependencies->new( 'Cartridges/DE_EPAGES/',
        ['DE_EPAGES::TLE'] );

    my $Dependencies = $Cartridge->getDependencies();

   # Checks if the only dependency (XML) of the DE_EPAGES::TLE cartridge is set.
    cmp_ok( $Dependencies->{'DE_EPAGES::TLE'}->[1], 'eq', 'DE_EPAGES::XML',
        'correct direct dependency' );

    # Checks if the init-color is correctly set.
    cmp_ok( $Dependencies->{'DE_EPAGES::TLE'}->[0], 'eq', 'lightgray',
        'correct node color init' );

    return;
}

sub TestGraph {

    my $Cartridge = DE_EPAGES::Diagnostics::API::DrawDependencies->new(
        'Cartridges/DE_EPAGES/', ['DE_EPAGES::TLE'], 'DE_EPAGES::TLE',
        ['DE_EPAGES::TLE'], 0 );

    my $Graph = $Cartridge->makeGraph();

    # Checks if a the base node is set.
    cmp_ok( index( $Graph, '<title>TLE</title>' ) , '!=', -1,
        'correct processing svg-image' );

    return;
}

sub Main {
    TestDependencies();
    TestGraph();
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

