/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.

	epages.html.buttonbarenhancer $Revision: 1.11 $

*/
dojo.provide("epages.html.buttonbarenhancer");
dojo.require("epages.html.element");
dojo.require("epages.browser");

epages.html.ButtonBarEnhancer = {

	buttonBars: [],
	allowUpdate: true,
	updateOccured: false,

	_selectedText: "",

	_resetButtonBar: function(buttonBarNode) {
		if(buttonBarNode.className.match("FixedButtonBar")) {
			dojo.removeClass(buttonBarNode, "FixedButtonBar");
			buttonBarNode.style.cssText = "";
			buttonBarNode.barPosition = null;
			buttonBarNode.parentNode.removeChild(buttonBarNode.barPlaceHolder);
			buttonBarNode.barPlaceHolder = null;
		}
	},

	_getRelevantForm: function(buttonBarNode) {
		for( var i=0,iLength=document.forms.length ; i<iLength ; i++ ) {
			if(	dojo.query(".Buttonbar", document.forms[i].parentNode)[0] ||
					// the class Buttonbar never has styles assigned to it
					dojo.query(".BatchActions", document.forms[i].parentNode)[0] ||
					dojo.query(".DialogButtonbar", document.forms[i].parentNode)[0] ||
					dojo.query(".WizardButtonBar", document.forms[i].parentNode)[0] ) {
				// parentNode should be the Dialog or at least the TabPage (both will do)
				return document.forms[i].parentNode;
			}
		}
		// there seems to be no form for that button bar (O.o ?) so we simply use the parentNode of it
		return buttonBarNode.parentNode;
	},

	init: function() {
		this.buttonBars = dojo.query(".DialogButtonbar", dojo.body()).concat(dojo.query(".BatchActions", dojo.body())).concat(dojo.query(".WizardButtonBar", dojo.body()));

		if(this.buttonBars.length) {
			dojo.connect(dojo.body(), "keyup", this, "checkScrollPosition");
			dojo.connect(window, "scroll", epages.html.ButtonBarEnhancer, "update");
			dojo.connect(window, "resize", epages.html.ButtonBarEnhancer, "update");
			dojo.connect(window.epagesDialog, "toggleArea", epages.html.ButtonBarEnhancer, "update");
			this.update();
		}
	},

	checkScrollPosition: function(evt) {
		var selectedNode = null;
		var selectedText = "someRandomStringThatWillNeverAppear0815x";

		if(evt) {
			selectedNode = evt.currentTarget;
		}else {
			if(document.selection && document.selection.createRange && document.selection.createRange().parentElement) {
				selectedNode = document.selection.createRange().parentElement();
				selectedText = document.selection.createRange().text;
				if(selectedNode.tagName == "BODY") {
					selectedNode = null;
					selectedText = null;
				}
			}else {
				var selectionObject;
				if(window.getSelection.rangeCount) {
					selectionObject = window.getSelection;
				}else {
					selectionObject = window.getSelection();
				}
				if(selectionObject.anchorNode) {
					selectedText = selectionObject.toString();
					selectedNode = selectionObject.anchorNode.offsetParent ? selectionObject.anchorNode : selectionObject.anchorNode.parentNode;
				}
			}
		}

		if(selectedNode && selectedText != this._selectedText) {

			this._selectedText = selectedText;

			var nodeYPosition = $E(selectedNode).getAbsolutePosition(dojo.body()).y ;
			var nodeYScrollPosition = nodeYPosition - (dojo.body().parentNode.scrollTop || window.scrollY);
			var windowHeight = dojo.body().parentNode.clientHeight;

			if(nodeYScrollPosition + 40 > windowHeight) {
				window.scrollTo(0, nodeYPosition - windowHeight + selectedNode.offsetHeight + 40);
			}
		}
	},

	update: function() {
		if(this.allowUpdate) {
			this.allowUpdate = false;
			this.updateOccured = false;
			dojo.forEach(this.buttonBars, dojo.hitch(this,
				function(buttonBar, entryIndex, array) {

					// hide emtpy button bars
					if(buttonBar.innerHTML.replace(/\s/g, "") == "") {dojo.addClass(buttonBar, "HideElement")}

					if(!buttonBar.className.match("HideElement")) {
						// a dialog area has changed
						if(buttonBar.dialogArea != window.actualDialogArea) {
							this._resetButtonBar(buttonBar);
							buttonBar.dialogArea = window.actualDialogArea;
						}

						// the dimensions of the parent node have changed (x2 = tolerance area)
						if(	buttonBar.parentNode.offsetHeight < (buttonBar.parentNode.oldOffsetHeight || 0) - buttonBar.offsetHeight * 2 ||
								buttonBar.parentNode.offsetHeight > (buttonBar.parentNode.oldOffsetHeight || 0) + buttonBar.offsetHeight * 2
						) {
							this._resetButtonBar(buttonBar);
							buttonBar.parentNode.oldOffsetHeight = buttonBar.parentNode.offsetHeight;
						}

						var correctTerm = (epages.Browser.engine == "WebKit")? dojo.body().scrollTop : 0;
						var barPosition = buttonBar.barPosition ? buttonBar.barPosition : new epages.html.Element(buttonBar).getAbsolutePosition(dojo.body()).y - correctTerm - 2;

						if(barPosition > (epages.Browser.engine == "WebKit" ? document.body : document.documentElement).scrollTop + (window.innerHeight || document.documentElement.clientHeight) - buttonBar.offsetHeight) {
							buttonBar.barPosition = barPosition;
							dojo.addClass(buttonBar, "FixedButtonBar");
							dojo.style(buttonBar, {
								left : (this._getRelevantForm(buttonBar).offsetLeft - (epages.Browser.engine == "WebKit" ? document.body : document.documentElement).scrollLeft) + "px",
								// 17 = substracted padding + border
								width : (this._getRelevantForm(buttonBar).offsetWidth - 17) + "px"
							});
							if(!buttonBar.barPlaceHolder) {
								buttonBar.barPlaceHolder = document.createElement("div");
								buttonBar.barPlaceHolder.style.height = (buttonBar.offsetHeight + 1) + "px";
								buttonBar.parentNode.insertBefore(buttonBar.barPlaceHolder, buttonBar.nextSibling);
							}
						}else {
							this._resetButtonBar(buttonBar);
						}
					}
				})
			);
			// we have to prevent to many calls of update() in a to small time span (mostly caused by window.scroll) - this significantly increases performance
			setTimeout(dojo.hitch(this, function(){
				this.allowUpdate = true;
				if(this.updateOccured){this.update()}
			}), 100);
		}else {
			this.updateOccured = true;
		}
	}
}

// initialize itself
dojo.addOnLoad(function() {
	setTimeout(dojo.hitch(epages.html.ButtonBarEnhancer, "init"), 100);
});