use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::FactFinder::API::Table::FactFinderShopMap qw (
    ExistsFactFinderShopMap
    InsertFactFinderShopMap
);
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval { TestInsertShopMap(); };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub TestInsertShopMap {
    my $Shop = LoadObjectByPath($ROOTPATH . '/TestShopFactFinder');
    ok(!ExistsFactFinderShopMap($Shop->id), 'map missing');
    InsertFactFinderShopMap({'ShopID' => $Shop->id});
    ok(ExistsFactFinderShopMap($Shop->id), 'map created');
    $Shop->delete;
    ok(!ExistsFactFinderShopMap($Shop->id), 'map gone');
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;