use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::FactFinder::API::Tools qw ( RemoveNamespacePrefixes );
use Test::More;

sub Main {
    testRemoveNamespacePrefixes();
    done_testing();
}

sub testRemoveNamespacePrefixes {
    _testSimpleArrayRef();
    _testSimpleHashRef();
    _testSimpleScalar();
    _testNestedStructure();
    _testNestedStructureWithPrefix();
    _testNestedStructureWithPrefix2();
}

sub _testSimpleArrayRef {
    my $INPUT = [ 'A', 'B' ];
    RemoveNamespacePrefixes( $INPUT );
    is_deeply( $INPUT , [ 'A', 'B' ], 'simple array ref' );
}

sub _testSimpleHashRef {
    my $INPUT = { 'A' => 'B' };
    RemoveNamespacePrefixes( $INPUT );
    is_deeply( $INPUT, { 'A' => 'B' }, 'simple hash ref' );
}

sub _testSimpleScalar {
    my $INPUT = 'ABC';
    RemoveNamespacePrefixes( $INPUT );
    is_deeply( $INPUT, 'ABC', 'simple scalar' );
}

sub _testNestedStructure {
    my $INPUT = { 'A' => [ { 'B' => 'C' } ] };
    RemoveNamespacePrefixes( $INPUT );
    is_deeply(
        $INPUT ,
        { 'A' => [ { 'B' => 'C' } ] },
        'nested structure'
    );
}

sub _testNestedStructureWithPrefix {
    my $INPUT = { 'ns1:A' => [ { 'ns2:B' => 'C' }, { 'D' => 'E' } ] } ;
    RemoveNamespacePrefixes($INPUT);
    is_deeply(
        $INPUT ,
        { 'A' => [ { 'B' => 'C' }, { 'D' => 'E' } ] },
        'nested structure with prefix'
    );
}

sub _testNestedStructureWithPrefix2 {
    my $INPUT = { 'A' => [ { 'B' => 'C'}, {'D' => 'E' } ], 'B' => [ 'ns1:abc' , 'test:nocheintest' , {'a_b:c_d' => 'e'} ] };
    RemoveNamespacePrefixes($INPUT);
    is_deeply(
        $INPUT,
        { 'A' => [ { 'B' => 'C'}, {'D' => 'E' } ], 'B' => [ 'ns1:abc' , 'test:nocheintest' , {'c_d' => 'e'} ] },
        'nested structure with prefix 2'
    );
}


RunScript( Sub => \&Main );
