use strict;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Dictionary::API::Template qw ( ProcessFileTemplate );
use DE_EPAGES::MeinPaket::API::Mail qw ( SendRegistrationReportMail );
use DE_EPAGES::MeinPaket::API::Object::RegisteredMerchant;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Locale qw ( GetInfoLocale );
use Getopt::Long;
use Test::More;

my $EMAIL;

GetOptions( 'mailto=s' => \$EMAIL, );

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            my $hReport = {
                '2010-01' => [
                    newMerchant( '2010', '01', 'TestShop1' )->tleHash,
                    newMerchant( '2010', '01', 'TestShop2' )->tleHash,
                    newMerchant( '2010', '01', 'TestShop3' )->tleHash,
                    newMerchant( '2010', '01', 'TestShop4' )->tleHash,
                ],
                '2010-02' => [
                    newMerchant( '2010', '01', 'TestShop5' )->tleHash,
                    newMerchant( '2010', '01', 'TestShop6' )->tleHash,
                    newMerchant( '2010', '01', 'TestShop7' )->tleHash,
                ],
            };

            testParseTemplate($hReport);
          SKIP: {
                skip 'dont send mail by default - add -mailto param.', 1 unless ($EMAIL);
                testSendRegistrationMail($hReport);
            }
            done_testing();
        }
    );
}

sub testSendRegistrationMail {
    my ($hReport) = @_;
    SendRegistrationReportMail( $hReport, $EMAIL );
    pass('sent mail.');
    return;
}

sub testParseTemplate {
    my ($hReport) = @_;

    my @ReportLoop;
    push @ReportLoop, { 'Month' => $_, 'NewAccounts' => $hReport->{$_} } foreach ( keys %$hReport );
    my $NOW = GetCurrentDateTime();

    my $System     = LoadRootObject();
    my $LocaleID   = $System->get('LocaleID');
    my $LanguageID = GetInfoLocale($LocaleID)->{LanguageID};

    my %Vars = (
        'Report' => \@ReportLoop,
        'NOW'    => $NOW,
        'INPUT'  => {
            Locale   => $LocaleID,
            Language => GetCodeByLanguageID($LanguageID),
        },
    );

    my $Content = ProcessFileTemplate(
        $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/MeinPaket/Templates/Mail/Mail-MeinPaketRegistrationReport-Content.html',
        $LanguageID, \%Vars
    );
    isnt( $Content, '', 'Content' );
    return;
}

sub newMerchant {
    my ( $Year, $Month, $ShopAlias ) = @_;
    my $Merch = DE_EPAGES::MeinPaket::API::Object::RegisteredMerchant->new();
    $Merch->set( { 'RegistrationYear' => $Year, 'RegistrationMonth' => $Month, 'ShopAlias' => $ShopAlias } );
    return $Merch;
}

RunScript( Sub => \&Main );

1;
