IF OBJECT_ID('i_Language') IS NOT NULL
    DROP PROCEDURE i_Language
GO
CREATE PROCEDURE i_Language (
    @LanguageID           int          = null
,   @IsUsed               bit          
,   @Code2                char(2)      = null
,   @Code3                char(3)      
,   @English              nvarchar(255) = null
,   @French               nvarchar(255) = null
,   @Native               nvarchar(255) = null
,   @DefaultLocaleID      nvarchar(5)  = null
,   @IsForBackoffice      bit          
,   @Dialect              char(2)      = null
) AS BEGIN
    DECLARE @TransactionCount int
    SELECT @TransactionCount = @@trancount

    IF @TransactionCount > 0 SAVE  TRANSACTION ti_language
    ELSE                     BEGIN TRANSACTION ti_language

        IF @LanguageID IS NULL SELECT @LanguageID = 1 + max(languageid) FROM language HOLDLOCK
        IF @LanguageID IS NULL SELECT @LanguageID = 1

        INSERT INTO language
               ( languageid, isused, code2, code3, english, french, native, defaultlocaleid, isforbackoffice, dialect)
        SELECT  @LanguageID,@IsUsed,@Code2,@Code3,@English,@French,@Native,@DefaultLocaleID,@IsForBackoffice,@Dialect

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION ti_language
            RETURN 1
        END

        SELECT @LanguageID

    IF @TransactionCount = 0
        COMMIT TRANSACTION ti_language
    RETURN 0
END
GO
