# perl -W
use strict;
use Test::More tests => 13;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath LoadRootObject
                                        LoadClassByAlias ExistsClassByAlias );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::JSONAttribute';
my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), "initialization" );

    my $TestObject = LoadObjectByPath( $ROOTPATH );
    my $Object = LoadClassByAlias('C1')->insertObject({
        Parent => $TestObject,
        Alias => 'C1O1'
    } );

    is_deeply( $Object->get('GoogleMap'), undef, 'no value set yet' );

    my $SollData = {
        'center' => '12345',
        'height' => '220',
        'width'  => '200',
        'key'    => {
            'subkey' => [
                'bla',
                'bli',
                'blubb',
            ]
        },
        'null'   => 'null',
        'undef'  => undef,
        'empty'  => '',
        'float'  => 123.456,
    };

    $Object->set({'GoogleMap'=> $SollData});

    my $IsData = $Object->get('GoogleMap');
    foreach my $Key ('center','height','widht','null','undef','empty','float') {
        is($IsData->{$Key}, $SollData->{$Key}, "key: $Key");
    }

    my $IsValue = $IsData->{'key'}{'subkey'}[2];
    my $SollValue = $SollData->{'key'}{'subkey'}[2];
    is($IsValue, $SollValue, "Data->{key}{subkey}[2]: $IsValue eq $SollValue");

    $Object->set({ 'GoogleMap' => ['a','b','c'] });
    is( $Object->get('GoogleMap')->[2], 'c', 'array ref' );

    eval { $Object->set({'GoogleMap' => undef}) };
    like( $@, qr/hash- or arrayref expected/, 'undef' );

    ok( Cleanup(), "cleanup" );

    DisconnectSite();
}

sub Cleanup {
    if( ExistsClassByAlias( 'C1' ) ) {
        LoadClassByAlias( 'C1' )->delete;
    }
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $ClassObject = LoadClassByAlias( 'Object' );
    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'GoogleMap', Type => 'Hash', Package => $ATTRIBUTE_PACKAGE });

    $ClassObject->insertObject( { Alias => 'Test', Parent => LoadRootObject() } );
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
