/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.rss
 *
 * @revision		$Revision: 1.10 $
 */

jQuery.plugin( 'de_epages.presentation.rss', ['de_epages.core', 'jQuery.class', 'ep.ajax'], function( $, undefined ){

	var mediaExpr = /(audio|image|video)/;

	$.Class( 'de_epages.presentation.Rss', {
		RSSCallbackFunc: null,
		Constructor: function( url, callbackFunc, fullPath ){
			//if fullPath is given we dont need to change the rss path
			this.RSSCallbackFunc = callbackFunc;
			this.requestFile(url,fullPath);
		},
		requestFile: function(url,fullPath){
			//check if file is already stored on server
			var fileURL = url;
			if(!fullPath){
				fileURL = '/WebRoot/RssFeeds/'+url.replace(/[^A-Za-z0-9]/g,"_")+'.xml';
			}

			ep.ajax({
				dataType:	'xml',
				url:		fileURL,
				type: 'GET',
				context:	this,
				success:	function(data, textStatus, XMLHttpRequest){
					this.readResponse(data);
				},
				error:		function(data, textStatus, XMLHttpRequest){
					this.requestFeed(url);
				}
			});
		},

		requestFeed: function(url){
			//file is not stored on server ask server to fetch it
			ep.ajax({
				dataType:	'xml',
				data: {
					'ObjectID' : ep.config.siteId,
					'ChangeAction': 'GetFeed',
					'feedurl': url
				},
				context:	this,
				success:	function(data, textStatus, XMLHttpRequest){
					this.readResponse(data);
				},
				error:		function(data, textStatus, XMLHttpRequest){
					//if the second request although fails we do nothing
				}
			});
		},

		readResponse: function(data){
			var itemname,
				subtitle,
				content,
				logo,
				date,
				xmlData = $(data);
			var feedData = {
				'version': null,
				'name': null,
				'description': null,
				'pagelink': null,
				'pubDate': null,
				'logo': {
					'src': null,
					'link': null,
					'title': null
				},
				'items': null
			};
			//rss feed type detection
			if(xmlData.find('rss').length){
				//RSS
				feedData.version = 'RSS';
				itemname = 'item';
				subtitle = 'description';
				content = 'description';
				logo = 'image';
				date = 'pubDate';
			}
			else if(xmlData.find('feed').length){
				//ATOM
				feedData.version = 'ATOM';
				itemname = 'entry';
				subtitle = 'subtitle';
				content = 'summary';
				logo = 'logo';
				date = 'updated';
			}
			//jquery has problems with xml namespaces see: http://bugs.jquery.com/ticket/155
			else if(xmlData.find('rdf\\:RDF').length || xmlData.find('RDF').length){
				//RDF
				feedData.version = 'RDF';
				itemname = 'item';
				subtitle = 'description';
				content = 'description';
				logo = 'image';
				//date = 'date'; //dc\\:date is the correct tag but we have to use only date because of the bug http://bugs.jquery.com/ticket/155
				date = (xmlData.find('dc\\:date')[0] !== undefined)? "dc\\:date" : "date";
			}
			else{
				//no valid feed
				return false;
			}

			//detect name of feed
			feedData.name = xmlData.find('title:first').text();
			//detect description
			var feedDesciption = xmlData.find(subtitle + ':first');
			feedData.description = feedDesciption.length ? feedDesciption.text() : xmlData.find('tagline:first').text();
			//detect link to feeds source page
			var firstLink = xmlData.find('link:first');
			feedData.pagelink = firstLink.text();
			if(!feedData.pagelink){
				var pageLink =  xmlData.find('link[rel="alternate"]:first');
				feedData.pagelink = pageLink.length ? pageLink.attr('href') : firstLink.attr('href');
			}
			//detect logo of feed
			var firstLogoNode = xmlData.find(logo+':first');
			var logoNode = firstLogoNode.length ? firstLogoNode : xmlData.find('icon:first');
			if(logoNode.length){
				var logoSrc = logoNode.find('url');
				feedData.logo.src = logoSrc.length ? logoSrc.text() : logoNode.text();
				if(!feedData.logo.src && logoNode[0].attributes[0]){
					feedData.logo.src = logoNode[0].attributes[0].value;
				}
				var logoTitle = logoNode.find('title');
				feedData.logo.title = logoTitle.length? logoTitle.text() : feedData.name;
				var logoLink = logoNode.find('link');
				feedData.logo.link = logoLink.length ? logoLink.text() : feedData.pagelink;
			}
			//detect date
			feedData.pubDate = xmlData.find(date + ':first').text();
			if(feedData.pubDate){
				var pareDateString = feedData.pubDate;
				feedData.pubDate = new Date(feedData.pubDate);
				//internet explorers date function can not pare ISO 8601 strings so we have to do it on our own
				if(isNaN(feedData.pubDate)){
					feedData.pubDate = new Date(pareDateString.substr(0,19).replace(/-/g,"/").replace(/[TZ]/g," "));
				}
			}
			var items = [];
			xmlData.find(itemname).each(function(index, elm){
				var item = {};
				var tmpElm = $(elm);
				//get title of news
				item.title = tmpElm.find('title:first').text();
				//get link of news
				if(feedData.version == 'ATOM'){
					var linkNode = tmpElm.find('link[rel="alternate"]:first');
					item.link = linkNode.length ? linkNode.attr('href') : tmpElm.find('link:first').attr('href');
				}
				else{
					var linkNode =tmpElm.find('link:first');
					item.link = linkNode.length ? linkNode.text() : feedData.pagelink;
				}
				//get content of news
				var textNode = tmpElm.find(content + ':first');
				item.text = textNode.length ? textNode.text() : tmpElm.find('content:first').text();
				if(!item.text){
					textNode = tmpElm.find('content\\:encoded:first');
					item.text = textNode.length ? textNode.text() : '';
				}
				else{
					textEncodedNode = tmpElm.find('content\\:encoded:first');
					item.textEncoded = textEncodedNode.length ? textEncodedNode.text(): tmpElm.find('encoded').text();
				}

				//get media elements of news
				item.audio = [];
				item.image = [];
				item.video = [];
				if(feedData.version == 'RSS'){
					tmpElm
						.find('enclosure')
						.each(function(){
							var elem = $(this),
								type = elem
									.attr('type')
									.replace(/\/.*$/,'');

							if( mediaExpr.test(type) ){
								item[ type ].push( elem.attr('url') );
							}
						});
				}

				//get pubDate if given
				var pubDateNode = tmpElm.find('pubDate:first');
				item.pubDate = pubDateNode.length ? pubDateNode.text() : tmpElm.find('published:first').text();
				if(item.pubDate){
					item.pubDate = new Date(item.pubDate);
				}
				items.push(item);
			});
			feedData.items = items;

			//give data to callback function
			this.RSSCallbackFunc(feedData);
			return true;
		}
	});

});