/*
	Copyright (c) 2006-2008, ePages GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.Favorites $Revision: 1.9 $

*/
dojo.provide("epages.cartridges.de_epages.presentation.widget.Favorites");
dojo.require("epages.widget");
dojo.require("epages.io.json");
dojo.require('epages.html');
dojo.require('epages.format');
dojo.require('epages.widget.LocalizedWidget');

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.Favorites",
	[epages.widget.LocalizedWidget],
	{
		objectId          : '',
		url               : '?',
		sidebarTabsId     : 'RightSidebarTabs',
		sidebarId         : 'RightSidebar',
		currentName       : '',
		currentURI        : '',

		_favorites        : undefined,
		_sidebarTabs      : undefined,
		_sidebar          : undefined,

		templatePath    : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/Favorites.html'),
		imagePath       : epages.themeUrl('images'),
		translationName : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/translation'),

		postCreate  : function() {
			this.inherited("postCreate", arguments);
			dojo.subscribe('Sidebar/selectTab', this, '_onTabSelect');
			if($$(this.sidebarId)){
				this._sidebar = $$(this.sidebarId);
			}
			if($$(this.sidebarTabsId)){
				this._sidebarTabs = $$(this.sidebarTabsId);
				var tab = this._sidebarTabs.currentTab();
				//check if tab is selected and sidebar is visible before loading
				if(tab && this._sidebar.isVisible){
					this._onTabSelect(tab);
				}
			}else{
				this._loadFavorites();
				this._updateLayout();
			}
		},

		_onTabSelect:function(tab){
			if(tab && tab.menu == 'favorites' && this._favorites == undefined){
				this._loadFavorites();
				this._updateLayout();
			}
		},

		_loadFavorites: function(){
			var currentFavorites = this._sendRequest('JSONGetFavorites');
			if(currentFavorites){
				this._favorites = currentFavorites.data;
				this._favorites.sort(function(a, b) {
					if (a.Name < b.Name){
						return -1;
					}
					if (a.Name > b.Name){
						return  1;
					}
					return 0; 
				});
				this.containerNode.innerHTML = '';
				for( var i=0,iLength=this._favorites.length ; i<iLength ; i++ ){
					var item = this.itemNode.cloneNode(true);
					item.style.display = '';
					var linkNode = epages.html.getElementsByClassName('ItemLink',item)[0];
					linkNode.innerHTML += epages.format.scrunch(this._favorites[i].Name,30,0.4);
					linkNode.title = this._favorites[i].Name;
					linkNode.href = this._favorites[i].URI;
					var deleteNode = epages.html.getElementsByClassName('ItemDeleteNode',item)[0];
					this.connect(deleteNode, 'onclick', 'deleteItem');
					this._favorites[i].itemNode = item;
					this._favorites[i].deleteNode = deleteNode;
					this.containerNode.appendChild(item);
				}
			}
		},

		_updateLayout: function(){
			if(this._favorites.length == 0){
				this.emptyNode.style.display = '';
				this.deleteNode.style.display = 'none';
				this.containerNode.style.display = 'none';
			}
			else{
				this.emptyNode.style.display = 'none';
				this.deleteNode.style.display = '';
				this.containerNode.style.display = '';
			}
			this.showAddToFavorites();
		},

		showAddToFavorites: function(){
			if(this._favorites){
				var exists = false;
				var existName="";
				for( var i=0,iLength=this._favorites.length ; i<iLength ; i++ ){
						if(this._favorites[i].Name == this.currentName){
							exists = true;
							existsName = this._favorites[i].Name;
					}
				}
				if(exists){
					this.existsNode.style.display = '';
					this.existsNodeTitle.innerHTML = epages.format.scrunch(existsName,30,0.4);
					this.addNode.style.display = 'none';
				}
				else{
					this.addNode.style.display = '';
					this.addNodeTitle.title = this.currentName;
					this.existsNode.style.display = 'none';
				}
			}
		},

		deleteItem: function(evt){
			originalTarget = evt.originalTarget || evt.srcElement;
			var favorite = undefined;
			var index = undefined;
			for( var i=0,iLength=this._favorites.length ; i<iLength ; i++ ){
				if(this._favorites[i].deleteNode == originalTarget){
					favorite = this._favorites[i];
					index = i;
				}
			}
			if(favorite){
				if(this._sendRequest('RemoveFavorite',{EventFavoriteID:favorite.EventFavoriteID})){
					this.containerNode.removeChild(favorite.itemNode);
					$A(this._favorites).remove(index);
					this._updateLayout();
				}
			}
		},

		_deleteFavorites: function(){
			if(this._sendRequest('DeleteFavorites')){
				this._favorites = [];
				this.containerNode.innerHTML = "";
				this._updateLayout();
			}
		},

		_sendRequest: function(changeAction,opt){
			var json = new epages.io.Json;
				var options = {
					'ViewAction':     'JSONViewResponse',
					'ChangeAction':   changeAction,
					'ObjectID':       this.objectId
				};
				if(opt){
					dojo.mixin(options,opt);
				}
				var result = json.loadSync(this.url,options);
				if(result.error){
					return false;
				}
				return result;
		},

		_addToFavorites:function(obj){
			var item = this.itemNode.cloneNode(true);
				item.style.display = '';
				var linkNode = epages.html.getElementsByClassName('ItemLink',item)[0];
				linkNode.innerHTML += epages.format.scrunch(obj.Name,30,0.4);
				linkNode.title = obj.Name;
				linkNode.href = obj.URI;
				var deleteNode = epages.html.getElementsByClassName('ItemDeleteNode',item)[0];
				this.connect(deleteNode, 'onclick', 'deleteItem');
				this.containerNode.appendChild(item);
				this._favorites.push({
					URI: obj.URI,
					Name: obj.Name,
					ObjectID: this.objectId,
					EventFavoriteID: obj.EventFavoriteID,
					itemNode: item,
					deleteNode: deleteNode
				});
				this._updateLayout();
		},

		_onClickAddNode: function(){
			var favoriteId = this._sendRequest('JSONAddFavorite',{URI: this.currentURI,Name: this.currentName});
			if(favoriteId != false){
				this._addToFavorites({
					URI  : this.currentURI,
					Name : this.currentName,
					EventFavoriteID : favoriteId.data.EventFavoriteID
				});
			}
		},

		addHistoryToFavorites: function(obj){
			var exists = false;
			if(this._favorites){
				for( var i=0,iLength=this._favorites.length ; i<iLength ; i++ ){
					if(this._favorites[i].URI == obj.URI){ 
						exists = true;
					}
				}
			}
			if(exists == false){
			//	var favoriteId = this._sendRequest('JSONCopyHistoryToFavorite',{EventHistoryID: obj.EventHistoryID});
				var favoriteId = this._sendRequest('JSONAddFavorite',{URI: obj.URI,Name: obj.Name});
				if(favoriteId != false){
					if(this._favorites){
						this._addToFavorites({
							URI  : obj.URI,
							Name : obj.Name,
							EventFavoriteID : favoriteId.data.EventFavoriteID
						});
					}
					return true;
				}
			}
			return false;
		}
});

