use strict;
use Test::More tests => 46;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadRootObject ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Presentation::API::PageType qw (ExistsPageTypeByAlias LoadPageTypeByAlias);

sub Main {
    ConnectTestSite();
    ok(Init(),'INIT');

    # insert base page type
    my $ClassSystem = LoadClassByAlias( 'System' );
    my $Class       = LoadClassByAlias( 'PageType' );
    my $oPageType1  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'TestBase',
        'PageTypeClass' => $ClassSystem,
    } );
    ok($oPageType1->id,'inserted PageTypeID Base '.$oPageType1->id);
    ok($oPageType1->get('PageTypeClass')->alias eq 'System', 'set the correct class');

    # insert sub page type
    my $oPageType2  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'SubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType1,
    } );
    ok($oPageType2->id,'inserted SubPageTypeID '.$oPageType2->id);

    my $oPageType4  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'XSubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType2,
    } );
    ok($oPageType4->id,'inserted SubPageTypeID '.$oPageType4->id);

    my $oPageType5  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'XXSubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType2,
    } );
    ok($oPageType5->id,'inserted SubPageTypeID '.$oPageType5->id);

    # insert sub page type
    my $oPageType3  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'SecSubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType1,
    } );
    ok($oPageType3->id,'inserted SecSubPageType '.$oPageType3->id);

    # exists name??
    ok( DE_EPAGES::Presentation::API::PageType::ExistsPageTypeByAlias($_),"$_ exists")
        foreach qw( TestBase SubPageType SecSubPageType XSubPageType XXSubPageType);

    # test basePageType
    my $Base = $oPageType2->get('BasePageType');
    ok( $Base,'BaseID for '. $oPageType2->alias .' is '. $Base->id .' ('.$Base->alias.')' );

    $oPageType4->set({'Alias'=>'XXXXXXXXXXSubPageType'});
    ok( $oPageType4->alias eq 'XXXXXXXXXXSubPageType', 'PageType Alias updated' );


    #my $Base = $oPageType1->basePageType();  #undef!!
    my $CartridgeID = GetPKeyCartridgeByPackage('DE_EPAGES::Presentation');

    $oPageType5->insertTemplate({'Name'=>'T0',
                                 'FileName'=>'System.Provider.default.TabPage.html',
                                 'CartridgeID'=>$CartridgeID,
                                 });
    ok($oPageType5->existsTemplate('T0'),'insert Template');

    my $Template1 = $oPageType5->insertTemplate({'Name'=>'T1',
                                 'FileName'=>'System.Provider1.default.TabPage.html',
                                 'CartridgeID'=>$CartridgeID,
                                 });
    ok($oPageType5->existsTemplate('T1'),'insert Template');

    # add some TemplateTypes for template T1
    my $TemplateTypeFolder = $oPageType5->folder('TemplateTypes')->insertChild('TemplateTypeFolder', {
        Alias => 'T1',
        Template => $Template1,
    });
    $TemplateTypeFolder->insertChild('TemplateType', {
        Alias => 'SubClass',
        FileName => 'T1-SubClass.html',
        CartridgeID => $CartridgeID,
        Template    => $Template1,
    });
    $TemplateTypeFolder->insertChild('TemplateType', {
        Alias => 'SubClassObject',
        FileName => 'T1-SubClassObject.html',
        CartridgeID => $CartridgeID,
        Template    => $Template1,
    });

    # add more templates
    $oPageType5->insertTemplate({
        'Name'=>'T2',
        'FileName'=>'System.Provider2.default.TabPage.html',
        'CartridgeID'=>$CartridgeID,
    });
    ok($oPageType5->existsTemplate('T2'),'insert Template');


    $oPageType3->insertTemplate({
        'Name'=>'T3',
        'FileName'=>'System.Provider3.default.TabPage.html',
        'CartridgeID'=>$CartridgeID,
    });
    ok($oPageType3->existsTemplate('T3'),'insert Template');

    # delete the template T2
    $oPageType5->deleteTemplate('T2');
    ok(!$oPageType5->existsTemplate('T2'),'delete Template T2');

    ## no critic qw(ProhibitLongLines)
    # attribut for T1
    my $hTemplate = $oPageType5->template('T1');
    ok( $hTemplate->{'Name'} eq 'T1', 'template name' );
    ok( $hTemplate->{'FileName'} eq 'System.Provider1.default.TabPage.html', 'template file name' );
    ok( $hTemplate->{'PageTypeID'} == $oPageType5->id, 'template page type' );

    # exists template for this page type ?
    ok( $oPageType5->existsTemplate($_),"$_ exists") foreach qw( T1 T0);
    ok( !$oPageType5->existsTemplate($_),"$_ does not exists") foreach qw( T3 T2);

    $oPageType5->updateTemplate('T1', undef, {
        'FileName' => 'MyCustomerTabPage.html'
    });
    ok( $oPageType5->template('T1')->{FileName} eq 'MyCustomerTabPage.html', 'updateTemplate' );

    # override the template for a sub class
    my $TestRoot = LoadObjectByPath('/Test');
    my $SubClass = $ClassSystem->insertSubClass({
        Alias => 'TestSubClass',
        Parent => $TestRoot,
    });
    my $SubClassObject = $SubClass->insertObject({
        Alias => 'SubClassObject',
        Parent => $TestRoot,
    });

# OBSOLETE
    $oPageType5->setTemplateFile( {
        Name => 'T1',
        Object => $SubClass,
        FileName => 'T1-SubClass.html',
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('T1', $SubClass)->{FileName} eq 'T1-SubClass.html', 'template changed for class' );
    ok( $oPageType5->template('T1', $SubClassObject)->{FileName} eq 'T1-SubClass.html', 'template changed for objects of class' );
    is( $oPageType5->templateTypeAlias( $SubClass, 'T1'), 'SubClass', 'templateTypeAlias SubClass' );

    # override the template for an object
    $oPageType5->setTemplateFile( {
        Name => 'T1',
        Object => $SubClassObject,
        FileName => 'T1-SubClassObject.html',
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('T1', $SubClass)->{FileName} eq 'T1-SubClass.html', 'template not changed for class' );
    ok( $oPageType5->template('T1', $SubClassObject)->{FileName} eq 'T1-SubClassObject.html', 'template changed for individual object' );
    is( $oPageType5->templateTypeAlias( $SubClassObject, 'T1'), 'SubClassObject', 'templateTypeAlias SubClassObject' );

    # restore class default
    $oPageType5->setTemplateFile( {
        Name => 'T1',
        Object => $SubClassObject,
        FileName => undef,
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('T1', $SubClassObject)->{FileName} eq 'T1-SubClass.html', 'class template restored for individual object' );

    # restore global default
    $oPageType5->setTemplateFile( {
        Name => 'T1',
        Object => $SubClass,
        FileName => undef,
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('T1')->{FileName} eq 'MyCustomerTabPage.html', 'global template restored 1' );
# /OBSOLETE

# NEW
    $oPageType5->setTemplateType($SubClass, 'T1', 'SubClass');
    ok( $oPageType5->template('T1', $SubClass)->{FileName} eq 'T1-SubClass.html', 'template changed for class' );
    ok( $oPageType5->template('T1', $SubClassObject)->{FileName} eq 'T1-SubClass.html', 'template changed for objects of class' );
    is( $oPageType5->templateTypeAlias( $SubClass, 'T1'), 'SubClass', 'templateTypeAlias SubClass' );

    # override the template for an object
    $oPageType5->setTemplateType($SubClassObject, 'T1', 'SubClassObject');
    ok( $oPageType5->template('T1', $SubClass)->{FileName} eq 'T1-SubClass.html', 'template not changed for class' );
    ok( $oPageType5->template('T1', $SubClassObject)->{FileName} eq 'T1-SubClassObject.html', 'template changed for individual object' );
    is( $oPageType5->templateTypeAlias( $SubClassObject, 'T1'), 'SubClassObject', 'templateTypeAlias SubClassObject' );

    # restore class default
    $oPageType5->setTemplateType($SubClassObject, 'T1', undef);
    ok( $oPageType5->template('T1', $SubClassObject)->{FileName} eq 'T1-SubClass.html', 'class template restored for individual object' );

    # restore global default
    $oPageType5->setTemplateType($SubClass, 'T1', undef);
    ok( $oPageType5->template('T1')->{FileName} eq 'MyCustomerTabPage.html', 'global template restored 1' );
# /NEW
    ## use critic

    my $Template = $oPageType5->getTemplate('T1');
    my $TemplateMap = $Template->getTemplateMap($SubClassObject);

    my $TemplateType = $TemplateMap->get('TemplateType');

    my $AttributeName = 'TestAttribute';
    $TemplateType->insertAttribute( {
        Alias => $AttributeName,
        Type => 'String',
        Length => 100,
        Package => 'DE_EPAGES::Presentation::API::Attributes::TemplateMap',
    } );

    ok( $TemplateType->existsAttribute($AttributeName), "TemplateMapStringAttribute: $AttributeName does exists." );

    $TemplateMap->set({$AttributeName=>'Blubb'});
    ok( $TemplateMap->get($AttributeName) eq 'Blubb', "$AttributeName has correct value." );

    ok(Cleanup(),'CLEANUP');
    DisconnectSite();
}


sub logger {
    my ($elem) = @_;
    use Data::Dumper qw (Dumper);
    print("ELEM:\n".Dumper( $elem )."\n");

    return 1;
}


sub Cleanup {

    LoadPageTypeByAlias('TestBase')->delete if ExistsPageTypeByAlias('TestBase');
    LoadPageTypeByAlias('SubPageType')->delete if ExistsPageTypeByAlias('SubPageType');
    LoadPageTypeByAlias('XSubPageType')->delete if ExistsPageTypeByAlias('XSubPageType');
    LoadPageTypeByAlias('XXSubPageType')->delete if ExistsPageTypeByAlias('XXSubPageType');
    LoadPageTypeByAlias('SecSubPageType')->delete if ExistsPageTypeByAlias('SecSubPageType');
    LoadPageTypeByAlias('XXXXXXXXXXSubPageType')->delete if ExistsPageTypeByAlias('XXXXXXXXXXSubPageType');
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');

    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);

1;