use strict;
use Test::Simple tests => 4;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory
    qw( LoadClassByAlias LoadRootObject ExistsObjectByPath LoadObjectByPath );

sub Init {
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    return 1;
}

sub Main {
    ConnectTestSite();
    ok( Init(), 'INIT' );

    # insert base page type
    my $ClassSystem  = LoadClassByAlias( 'System' );
    my $Class        = LoadClassByAlias( 'PageType' );
    my $TestPageType = LoadRootObject()->folder( 'PageTypes' )->insertChild(
        $Class,
        {   'Alias'         => 'TestBase',
            'PageTypeClass' => $ClassSystem,
        }
    );

    # insert template to pagetype
    my $CartridgeID   = GetPKeyCartridgeByPackage( 'DE_EPAGES::Presentation' );
    my $ImageTemplate = $TestPageType->insertTemplate(
        {   'Name'        => 'Image',
            'FileName'    => 'TestBase.Image.html',
            'CartridgeID' => $CartridgeID,
        }
    );

    my $TemplateTypes      = $TestPageType->folder( 'TemplateTypes' );
    my $TemplateTypeFolder = $TemplateTypes->insertChild(
        'TemplateTypeFolder',
        {   'Alias'     => 'Image',
            'IsVisible' => 1,
            'Position'  => 10,
            'Template'  => $ImageTemplate,
        }
    );

    $TemplateTypeFolder->insertChild(
        'TemplateType',
        {   'Alias'       => 'DefaultImage',
            'Position'    => 10,
            'Template'    => $ImageTemplate,
            'FileName'    => 'TestBase.Image-DefaultImage.html',
            'CartridgeID' => $CartridgeID,
        }
    );

    my $Coverflow = $TemplateTypeFolder->insertChild(
        'TemplateType',
        {   'Alias'       => 'Coverflow',
            'Position'    => 20,
            'Template'    => $ImageTemplate,
            'FileName'    => 'TestBase.Image-Coverflow.html',
            'CartridgeID' => $CartridgeID,
        }
    );

    $Coverflow->insertAttribute(
        {   Alias   => 'BackgroundColor',
            Type    => 'String',
            Package => 'DE_EPAGES::Presentation::API::Attributes::TemplateMap',
        }
    );
    ok( $Coverflow->existsAttribute( 'BackgroundColor' ), 'additional Attribute exists' );

    my $TestProduct = LoadRootObject()->insertChild( 'Object', { Alias => 'TestProduct' } );
    my $TemplateMap = $Coverflow->get('Template')->setTemplateMap($TestProduct, $Coverflow);

    $TemplateMap->set({ 'BackgroundColor' => 'white' });
    ok( $TemplateMap->get('BackgroundColor') eq 'white', 'BackgroundColor has correct value.' );

    ok( Cleanup(), 'CLEANUP' );
    DisconnectSite();
}

sub logger {
    my ( $elem ) = @_;
    use Data::Dumper qw (Dumper);
    print( "ELEM:\n" . Dumper( $elem ) . "\n" );

    return 1;
}

sub Cleanup {
    LoadObjectByPath( '/Test' )->delete if ExistsObjectByPath( '/Test' );
    LoadObjectByPath( '/PageTypes/TestBase' )->delete
        if ExistsObjectByPath( '/PageTypes/TestBase' );
    LoadObjectByPath( '/TestProduct' )->delete if ExistsObjectByPath( '/TestProduct' );

    return 1;
}

RunScript( Sub => \&Main, Debug => 1 );

1;
