use strict;
use Test::Simple tests => 6;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory
    qw( LoadClassByAlias LoadRootObject ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

sub Init {
    Cleanup();

    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );

    return 1;
}

sub Main {
    ConnectTestSite();
    ok( Init(), 'INIT' );

    # insert base page type
    my $ClassSystem  = LoadClassByAlias( 'System' );
    my $Class        = LoadClassByAlias( 'PageType' );
    my $TestPageType = LoadRootObject()->folder( 'PageTypes' )->insertChild(
        $Class,
        {   'Alias'         => 'TestBase',
            'PageTypeClass' => $ClassSystem,
        }
    );

    # insert template to pagetype
    my $CartridgeID   = GetPKeyCartridgeByPackage( 'DE_EPAGES::Presentation' );
    $TestPageType->insertTemplate(
        {   'Name'        => 'Image',
            'FileName'    => 'TestBase.Image.html',
            'CartridgeID' => $CartridgeID,
        }
    );

    ImportObjects("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Presentation/t/Coverflow.xml");
    ok (defined $TestPageType, 'Import OK');

    my $Coverflow = $TestPageType->getTemplate('Image')->getTemplateType('Coverflow');

    ok (defined $Coverflow, 'Coverflow templatetype from import xml found');
    ok( $Coverflow->existsAttribute( 'BackgroundColor' ), 'additional Attribute exists' );

    my $TestProduct = LoadRootObject()->insertChild( 'Object', { Alias => 'TestProduct' } );
    my $TemplateMap = $Coverflow->get('Template')->setTemplateMap($TestProduct, $Coverflow);

    $TemplateMap->set( { 'BackgroundColor' => 'blue' } );
    ok( $TemplateMap->get( 'BackgroundColor' ) eq 'blue', 'Get attribute' );


    ok( Cleanup(), 'CLEANUP' );
    DisconnectSite();
}

sub logger {
    my ( $elem ) = @_;
    use Data::Dumper qw (Dumper);
    print( "ELEM:\n" . Dumper( $elem ) . "\n" );

    return 1;
}

sub Cleanup {
    LoadObjectByPath( '/Test' )->delete if ExistsObjectByPath( '/Test' );
    LoadObjectByPath( '/PageTypes/TestBase' )->delete if ExistsObjectByPath( '/PageTypes/TestBase' );
    LoadObjectByPath( '/TestProduct' )->delete if ExistsObjectByPath( '/TestProduct' );

    return 1;
}

RunScript( Sub => \&Main, Debug => 1 );

1;
