use strict;
use Test::Simple tests => 7;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadRootObject ExistsClassByAlias ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Presentation::API::PageType qw( ExistsPageTypeByAlias LoadPageTypeByAlias );


sub Init {
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

sub Main {
    ConnectTestSite();
    ok(Init(),'INIT');

    # insert base page type
    my $ClassSystem = LoadClassByAlias( 'System' );
    my $Class = LoadClassByAlias( 'PageType' );
    my $TestPageType  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'         =>'TestBase',
        'PageTypeClass' => $ClassSystem,
    } );
    ok( $TestPageType->id,                                       'inserted PageTypeID Base '.$TestPageType->id );
    ok( $TestPageType->get('PageTypeClass')->alias eq 'System',  'set the correct class' );


    # insert template
    my $CartridgeID = GetPKeyCartridgeByPackage('DE_EPAGES::Presentation');
    my $Template = $TestPageType->insertTemplate({
        'Name'          =>'ProviderDefault',
        'FileName'      =>'System.Provider.default.TabPage.html',
        'CartridgeID'   =>$CartridgeID
    } );

    my $TestTemplateType = $Template->getDefaultTemplateType;
    ok( defined $TestTemplateType,                                                    'Template type was found by template name' );
    ok( $TestTemplateType->get('FileName') eq 'System.Provider.default.TabPage.html', 'Filename stored in template type' );
    ok( $TestTemplateType->get('CartridgeID') eq $CartridgeID,                        'CartridgeID stored in template type' );


#    # new coverflow class
#    my $CoverflowClass = $TemplateTypeClass->insertSubClass({
#        Alias => 'Coverflow',
#        Package => 'DE_EPAGES::Object::API::Object::Object',
#    });
#
#    #insert attribute to coverflow
#    unless ($CoverflowClass->existsAttribute( "BackgroundColor" )) {
#         $CoverflowClass->insertAttribute( {
#                Alias => "BackgroundColor",
#                Type => "Integer",
#                Package => "DE_EPAGES::Object::API::Attributes::DefaultAttribute"
#            });
#    }
#
#    ok($CoverflowClass->existsAttribute( "BackgroundColor" ), "additional Attribute exists");
#
#
#    my $CoverflowObject = $TemplateTypeFolder->insertChild('Coverflow', {
#        Alias => 'CoverflowTemplate',
#        Position => 40,
#        FileName => 'SF/SF-Product.Content-Base-CoverFlow_Color1.html',
#        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
#        PageType =>  $TestPageType,
#    });
#
#
#
#
#    $CoverflowObject->set({ 'BackgroundColor' => 12 });
#
#
#     my $Obj = LoadObjectByPath('/Test/PageTypes/TestPageType/TemplateTypes/Content-Base/CoverflowTemplate');
#
#   # print $CoverflowObject->get('BackgroundColor') ;
#    ok($Obj->get('BackgroundColor') eq 12, "Get attribute");

    ok(Cleanup(),'CLEANUP');
    DisconnectSite();
}


sub logger {
    my ($elem) = @_;
    use Data::Dumper qw (Dumper);
    print("ELEM:\n".Dumper( $elem )."\n");

    return 1;
}


sub Cleanup {
    LoadClassByAlias('Coverflow')->delete if ExistsClassByAlias('Coverflow');
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');
    LoadPageTypeByAlias('TestBase')->delete if ExistsPageTypeByAlias('TestBase');
    return 1;
}


RunScript(Sub=>\&Main, Debug=>1);

1;