/* -*- PERL -*-*/
%{
    #========================================================================================
    # package	DE_EPAGES::TLE::API::Parser
    # state	private
    #----------------------------------------------------------------------------------------
    # description   This module is created by Perl yacc. see tle.y.
    #========================================================================================
use base Exporter;

use strict;
our @EXPORT_OK = qw(
    InternalCompile
    InternalAnalyze
);

use DE_EPAGES::TLE::API::LexConstants;
use DE_EPAGES::TLE::API::Lexer      qw ( Lexer );
use DE_EPAGES::TLE::API::Execute    qw ( BuildTree Tree2ByteCode );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );

sub ErrorDefinition { return {
    'GRAMMAR_ERROR' => 'Grammar error at line (#Line) #Message',
    'TLE_ERROR'     => 'TLE error at line (#Line) #Message',
}}

    # to recognize possible open strings
    # assumption is that most string constants are on one line
    # gives additional error descr if we detect a syntax error after a started newline
    # needs careful reset if we have recognized a syntactical valid construct
my %newlineInString = ('newlineInString'=>0, 'startAtLineNo'=> 0,);

%}
%token  BODY TEMPLATE
        IF ELSIF ELSE ENDIF
        NUMCONSTANT STRCONSTANT VARIABLE
        FUNCTION FUNCTIONNAME
        BLOCK ENDBLOCK LOOP ENDLOOP JOIN ENDJOIN LOCAL ENDLOCAL WITH ENDWITH MENU ENDMENU NAVBAR ENDNAVBAR WITH_LANGUAGE ENDWITH_LANGUAGE WITH_ERROR ENDWITH_ERROR
        PARENTHESE LIST SPACE FORMAT
        PERLSCRIPT JSSCRIPT END_JSSCRIPT

%left OR
%left AND
%left NEQ NNE
%left EQ NE
%left NLT NLE NGT NGE
%left IN
%left '+' '-'
%left '*' '/' '%'
%left '.'
%right NOT SPACE

%start start

%%

start           :   template                    { $s->{'tree'} = $$; }
                ;
template        :                               { $$ = undef; }
                |   template templatebody       { $$ = BuildTree($TEMPLATE, [$1,$2], $1, $2); }
                ;
templatebody    :   PERLSCRIPT
                |   formvariable
                |   if
                |   block
                |   function
                |   body                        { $$ = BuildTree($BODY, [$1], $$); }
                |   BODY
                ;
body            :   '+'|'-'|'*'|'/'|'%'|'('|')'|'.'|','
                |   SPACE
                |   NUMCONSTANT
                |   STRCONSTANT
                |   NLT
                |   NGT
                ;
if              :   IF '(' expr ')' template elsif else ENDIF
                                                { $$ = BuildTree($IF, [$1,$2,$3,$4,$5,$6,$7,$8], $3, $5, $6, $7); }
                ;
else            :                               { $$ = undef; }
                |   ELSE template               { $$ = BuildTree($ELSE, [$1,$2], $2); }
                ;
elsif           :                               { $$ = undef; }
                |   ELSIF '(' expr ')' template elsif
                                                { $$ = BuildTree($ELSIF, [$1,$2,$3,$4,$5,$6], $3, $5, $6); }
                ;
block           :   BLOCK '(' STRCONSTANT ',' expr_list ')' template ENDBLOCK
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6,$7,$8], $3, $5, $7); }
                |   JOIN '(' expr_list ')' template ENDJOIN
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], 'JOIN', $3, $5 ); }
                |   LOOP '(' expr_list ')' template ENDLOOP
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], 'LOOP', $3, $5 ); }
                |   WITH '(' expr_list ')' template ENDWITH
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], 'WITH', $3, $5 ); }
                |   LOCAL '(' STRCONSTANT ',' expr ')' template ENDLOCAL
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6,$7,$8], 'LOCAL', (BuildTree($LIST, [$3,$4,$5], $3, $5))[1], $7); }
                |   MENU '(' expr_list ')' template ENDMENU
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], 'MENU', $3, $5 ); }
                |   NAVBAR '(' expr_list ')' template ENDNAVBAR
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], 'NAVBAR', $3, $5 ); }
                |   WITH_LANGUAGE '(' expr_list ')' template ENDWITH_LANGUAGE
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], 'WITH_LANGUAGE', $3, $5 ); }
                |   WITH_ERROR '(' expr_list ')' template ENDWITH_ERROR
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], 'WITH_ERROR', $3, $5 ); }
                |   JSSCRIPT template END_JSSCRIPT
                                                { $$ = BuildTree($JSSCRIPT, [$1,$2,$3], $1, $2, $3); }
                ;
function        :   FUNCTION '(' STRCONSTANT ',' expr_list ')'
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4,$5,$6], $3, (BuildTree($LIST, [$5], $5))[1], undef); }
                |   FUNCTIONNAME '(' expr_list ')'
                                                { $$ = BuildTree($BLOCK, [$1,$2,$3,$4], $1->{'parameter'}, $3, undef); }
                ;
expr_list       :   expr                        { $$ = BuildTree($LIST, [$1], $1); }
                |   expr_list ',' expr          { $$ = BuildTree($LIST, [$1,$2,$3], $1, $3); }
                ;
expr            :   binop
                |   unop
                |   formvariable
                |   NUMCONSTANT
                |   STRCONSTANT
                |   function
                |   '(' expr  ')'               { $$ = BuildTree($PARENTHESE, [$1,$2,$3], $2); }
                |   expr SPACE                  { $$ = BuildTree(undef, [$1,$2],    $1); }
                |   SPACE expr                  { $$ = BuildTree(undef, [$1,$2],    $2); }
                ;
binop           :   expr OR  expr               { $$ = BuildTree($OR,  [$1,$2,$3], $1, $3); }
                |   expr AND expr               { $$ = BuildTree($AND, [$1,$2,$3], $1, $3); }
                |   expr EQ  expr               { $$ = BuildTree($EQ,  [$1,$2,$3], $1, $3); }
                |   expr NE  expr               { $$ = BuildTree($NE,  [$1,$2,$3], $1, $3); }
                |   expr NEQ expr               { $$ = BuildTree($NEQ, [$1,$2,$3], $1, $3); }
                |   expr NNE expr               { $$ = BuildTree($NNE, [$1,$2,$3], $1, $3); }
                |   expr NLT expr               { $$ = BuildTree($NLT, [$1,$2,$3], $1, $3); }
                |   expr NLE expr               { $$ = BuildTree($NLE, [$1,$2,$3], $1, $3); }
                |   expr NGT expr               { $$ = BuildTree($NGT, [$1,$2,$3], $1, $3); }
                |   expr NGE expr               { $$ = BuildTree($NGE, [$1,$2,$3], $1, $3); }
                |   expr IN  expr               { $$ = BuildTree($IN,  [$1,$2,$3], $1, $3); }
                |   expr '+' expr               { $$ = BuildTree(ord('+'), [$1,$2,$3], $1, $3); }
                |   expr '-' expr               { $$ = BuildTree(ord('-'), [$1,$2,$3], $1, $3); }
                |   expr '*' expr               { $$ = BuildTree(ord('*'), [$1,$2,$3], $1, $3); }
                |   expr '/' expr               { $$ = BuildTree(ord('/'), [$1,$2,$3], $1, $3); }
                |   expr '%' expr               { $$ = BuildTree(ord('%'), [$1,$2,$3], $1, $3); }
                |   expr '.' expr               { $$ = BuildTree(ord('.'), [$1,$2,$3], $1, $3); }
                ;
unop            :   NOT expr                    { $$ = BuildTree($NOT,     [$1,$2], $2); }
                |   '-' expr                    { $$ = BuildTree(ord('-'), [$1,$2], $2); }
                |   '+' expr                    { $$ = BuildTree(ord('+'), [$1,$2], $2); }
                ;
formvariable    :   VARIABLE format             { $$ = BuildTree($VARIABLE, [$1,$2], $1, $2); }
                ;
format          :                               { $$ = undef; }
                |   FORMAT
                ;
%%

# error interface has changed a bit
# the second argument is the reference argument passed to yyparse as argument
sub yyerror {
    my ($msg, $s) = @_;
    Error('TLE_ERROR', {
        'Message'   => $msg,
        'Line'      => $s->{'lineno'},
        'yyparse'   => $s->{'text'},
    });
}
## use critic
#========================================================================================
# function     InternalCompile
#----------------------------------------------------------------------------------------
# syntax       my $ByteCode = InternalCompile($rPlainText);
#----------------------------------------------------------------------------------------
# description  This procedure parses the given template and creates a perl execution string.
#----------------------------------------------------------------------------------------
# input        $rPlainText | template | ref.string
# return       $ByteCode | the complied template | string
#========================================================================================
sub InternalCompile ($) {
    my ($rPlainText) = @_;
    return Tree2ByteCode({ 'tag' => $BODY, 'text'=> '' }) unless defined $$rPlainText && length($$rPlainText);

    # prepare parser variables
    $yydebug = 0;
    my $s = {
        'text'             => $$rPlainText,
        'lineno'           => 1,
        'lexer_last_token' => $BODY,
        'lexer_context'    => 0,
    };
    my $p = new( __PACKAGE__, \&Lexer,  \&yyerror, $yydebug );
    $p->yyparse($s);
    return Tree2ByteCode($s->{'tree'});
}

#========================================================================================
# function     InternalAnalyze
#----------------------------------------------------------------------------------------
# syntax       my $Error = InternalAnalyze($rPlainText);
#----------------------------------------------------------------------------------------
# description  This procedure parses the given template returns the error.
#----------------------------------------------------------------------------------------
# input        $rPlainText | template | ref.string
# return       $Error | error object (or undef) | object
#========================================================================================
sub InternalAnalyze ($) {
    my ($rPlainText) = @_;
    return undef unless length($$rPlainText);

    # prepare parser variables
    $yydebug = 0;
    my $s = {
        'text'             => $$rPlainText,
        'lineno'           => 1,
        'lexer_last_token' => $BODY,
        'lexer_context'    => 0,
    };
    my $p = new( __PACKAGE__, \&Lexer,  \&yyerror, $yydebug );
    eval {
        $p->yyparse($s);
    };
    return GetError() if ExistsError();
    return undef;
}
