use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle GetTestSite RunOnStore );
use Test::More;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            my $dbi = GetCurrentDBHandle();
            plan skip_all => 'Test only working on mysql' unless ( $dbi->driverName eq 'mysql' );

            # tests
            _testInvalidAutoIncrement($dbi);
            done_testing();
        }
    );
}

sub _testInvalidAutoIncrement {
    my ($dbi) = @_;

    my $sql = << 'EOL';
select distinct c.table_name, c.column_name, kcu2.CONSTRAINT_NAME, kcu2.REFERENCED_TABLE_NAME
from columns c, key_column_usage kcu, key_column_usage kcu2
where c.extra="AUTO_INCREMENT"
and c.table_schema=kcu.table_schema
and c.table_name=kcu.table_name
and kcu.constraint_name="PRIMARY"
and kcu.column_name=kcu2.column_name
and c.table_schema=kcu2.table_schema
and c.table_name=kcu2.table_name
and kcu2.REFERENCED_TABLE_NAME is not null
and c.table_schema=?
EOL

    my $dbname = $dbi->databaseName();
    $dbi->do('use INFORMATION_SCHEMA');
    my $aaRows = $dbi->execute( $sql, [$dbname], ['String'] );
    if ( !@$aaRows ) {
        pass('no autoincrement on tables derived from object');
    }
    else {
        foreach my $aRow (@$aaRows) {
            fail(
                'derived table ' . $aRow->[0] . ' has AUTO_INCREMENT on column ' . $aRow->[1] . ' - must be removed.' );
        }
    }
}

RunScript( Sub => \&Main );

1;
