use Test::More;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );


plan tests => 10;

sub Main {
    my $URL = 'http://localhost/epages/TestRequest.sf';
    my $FakeRequest = MyFakeRequest->new;
    my $Response = $FakeRequest->get( $URL.'?PerformanceBar=1' );

    cmp_ok( $Response->code, '==', 200, '200 OK' );
    cmp_ok( $Response->content, 'eq', 'TestRequest: OK', 'content' );
    my @cookie = split(/, /  ,$Response->header('Set-Cookie'));
    my %cookieValues;
    foreach my $setCookie (@cookie)
    {

        if($setCookie=~/^(.*?)=(.*?);/x){
            $cookieValues{$1} = $2;
        }
    }

    ok($cookieValues{'PerformanceBar'} eq 'enabled' ,'PerformanceBar is enabled');
    cmp_ok($cookieValues{'RequestTime'},'<',0.009,'RequestTime is set and smaller then 0.009');
    ok(defined($cookieValues{'TLETime'}),'TLE Value is set');
    ok(defined($cookieValues{'MysqlTime'}),'Mysql Value is set');
    cmp_ok($cookieValues{'AppServerMemory'},'>',0,'Application Server Memory Usage is set and bigger then 0');

    $Response = $FakeRequest->get( $URL.'?PerformanceBar=0' );

    @cookie = split(/, /  ,$Response->header('Set-Cookie'));
    %cookieValues = ();
    foreach my $setCookie (@cookie)
    {
        if($setCookie=~/^(.*?)=(.*?);/x){
            $cookieValues{$1} = $2;
        }
    }

    cmp_ok($cookieValues{'PerformanceBar'},'eq','','PerformanceBar is disabled');
    ok(!defined($cookieValues{'TLETime'}),'TLE Value is not set');
    ok(!defined($cookieValues{'MysqlTime'}),'Mysql Value is not set');

    return;
}

RunScript(Sub=>\&Main, Debug=>1);

package MyFakeRequest;
use base qw( DE_EPAGES::WebInterface::API::FakeRequest );

sub lookupPoolDB {
    my $self = shift;
    my ($Request) = @_;

    # set the X-Headers
    $Request->header('X-ePages-Site' => 'DummyGUID');
    $Request->header('X-ePages-Store' => 'DummyStore');
    $Request->header('X-ePages-Pool' => 'DummyPool');
    return;
}
