/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.preload
 *
 * @revision		$Revision: 1.5.46.1 $
 */

jQuery.plugin( 'ep.preload', 'ep.core', function( $, undefined ){

	var files = {};
	var preloadingStack = [];

	var running = false;
	var preloadingFinished = false;
	var sortAlgorithm = function(a, b) {
		//sort stack by priority
		return a.priority - b.priority;
	};

	var start = function(){
		//build sorted preloadingStack
		for(var key in files){
			preloadingStack.push({
				"file" : key,
				"priority" : files[key]
			});
		}
		preloadingStack.sort(sortAlgorithm);
		if(!running){
			running = true;
			preloadFile();
		}
	};

	var preloadFile = function(currentFile){
		if(!currentFile){
			if(preloadingStack.length){
				currentFile = preloadingStack.shift().file;
			}
			else{
				return;
			}
		}
		if(running){
			$.ajax({
				"cache" : true,
				"url" : currentFile,
				"success" :  function(){
					if(preloadingStack.length && !preloadingFinished){
						preloadFile();
					}
					else{
						preloadingFinished = true;
					}
				},
				"error" : function(){
					$.debug.warn('ep.preload: unable to load ', currentFile);
					if(preloadingStack.length && !preloadingFinished){
						preloadFile();
					}
					else{
						preloadingFinished = true;
					}
				}
			});
		}
	};

	ep.preload = function(file, priority){
		if(preloadingFinished){
			//if all files are loaded we load the next file directly
			preloadFile(file);
		}
		else{
			//add file to files hash
			if(!files[file]){
				files[file] = priority;
			}
		}
	};

	$.ready(function(){
		// delay preload to prevent blocking other methods on DOM ready
		window.setTimeout(function(){
				start();
			},100);
	});

});