use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MeinPaket::API::DTO::CatalogStructure;
use DE_EPAGES::MeinPaket::API::DTO::DownloadRequest;
use DE_EPAGES::MeinPaket::API::DTO::Header;
use DE_EPAGES::MeinPaket::API::Object::RemoteMeinPaket;
use DE_EPAGES::MeinPaket::API::XML::ResponseHandler;
use DE_EPAGES::MeinPaket::t::TestHelper qw ( RunWithRemoteCall );
use Test::More;

sub testDownloadCategories {
    my $Remote = DE_EPAGES::MeinPaket::API::Object::RemoteMeinPaket->new(
        'ServiceURL'    => 'http://mp-api.mepa-home.de/dealerapi/xml',
        'IsSandbox'     => 1,
        'Logging'       => 1,
    );

    my $Header = DE_EPAGES::MeinPaket::API::DTO::Header->new(
        login    => '3rdparty-test@epages.com',
        password => 'MeinPaket2012',
        language => 'de',
    );


    my $Request = DE_EPAGES::MeinPaket::API::DTO::DownloadRequest->new(
        'header'           => $Header,
        'catalogstructure' => DE_EPAGES::MeinPaket::API::DTO::CatalogStructure->new(),
    );

    my $Response = $Remote->postRequest($Request, { ReturnXML => 1 });
    cmp_ok( length $Response, '>', 1000, 'xml reponse at least 1k string' );

    my $Result = DE_EPAGES::MeinPaket::API::XML::ResponseHandler->importXML($Response);
    isa_ok( $Result, 'DE_EPAGES::MeinPaket::API::DTO::DownloadResponse' );

    cmp_ok( scalar keys %{ $Result->CategoryHash() }, '>', 100, 'at least 100 categories' );

    # umlaut test.
    my $Category1_1_21 = $Result->getCategory('1.1.21');
    is($Category1_1_21->name(), 'Business & Brse', '1.1.21 category name');
}

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            RunWithRemoteCall(sub {
                testDownloadCategories();
            });
            done_testing();
        }
    );
}

RunScript( Sub => \&Main );

1;
