use strict;
use Test::More tests => 10;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );


sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $Shop = LoadRootObject()->folder('Shops')->child( 'TestShop' );
    ok( $Shop->id, 'create shop' );

    $Shop->featureMaxValue('Newsletter', 1);

    my $CampaignFolder = $Shop->folder('NewsletterCampaigns');
    my $Campaign = $CampaignFolder->insertChild('NewsletterCampaign', {
        'Alias' => 'TestCampaign',
        'Name' => 'TestCampaign'
    });
    ok( $Campaign->id, 'create campaign' );

    my $NewsletterFolder = $Campaign->folder('Newsletters');
    my $Newsletter = $NewsletterFolder->insertChild('Newsletter', {'Alias' => 'TestNewsletter', 'Name' => 'TestNewsletter'});
    ok( $Newsletter->id, 'create newsletter' );

    my $ExistsAddressee = $Campaign->existsAddressee('TestEMail@epages.com');
    ok( !$ExistsAddressee, 'no addressee' );

    my $Addressee = $Campaign->addAddressee({'Name' => 'TestAddressee', 'EMail' => 'TestEMail@epages.com'});
    ok( $Addressee->id, 'create addressee' );

    $ExistsAddressee = $Campaign->existsAddressee('TestEMail@epages.com');
    ok( $ExistsAddressee, 'exists addressee' );

    my $Recipient = $Newsletter->addRecipient({'Name' => 'TestNewsletter', 'EMail' => 'TestEMail@epages.com'});
    ok( $Recipient->id, 'create recipient' );

    #Create user
    my $user = $Shop->folder('Users')->insertChild( 'User', { 'Alias' => 'TestUser1', 'Password' => 'XXX', 'EMail' => 'TestUser@epages.com' } );

    $Addressee = $Campaign->addAddressee({'User' => $user, 'EMail' => 'TestUser@epages.com', 'Name' => 'TestAddressee'});
    is( $Addressee->get('User')->id, $user->id,'added user to campaign' );



    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    my $ShopFolder = LoadRootObject()->folder('Shops');
    $ShopFolder->child('TestShop')->delete if  $ShopFolder->existsChild('TestShop');
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->folder('Shops')->insertChild( 'Shop', { 'Alias' => 'TestShop' } );
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
